package com.zanxiang.common.domain;

import com.zanxiang.common.enums.HttpStatusEnum;

import java.util.HashMap;
import java.util.Map;

/**
 * 返回对象
 *
 * @author xufeng
 * @date 2022/6/9 10:41
 */
public class ResultMap extends HashMap<String, Object> {

    private static final long serialVersionUID = 1L;

    public ResultMap() {
        put("code", 0);
    }

    public static ResultMap error() {
        return error(500, "未知异常，请联系管理员");
    }

    public static ResultMap error(String msg) {
        return error(500, msg);
    }

    public static ResultMap error(int code, String msg) {
        ResultMap r = new ResultMap();
        r.put("code", code);
        r.put("msg", msg);
        return r;
    }

    public static ResultMap ok() {
        ResultMap r = new ResultMap();
        r.put("msg", HttpStatusEnum.SUCCESS.getMsg());
        return r;
    }

    public static ResultMap ok(Object data) {
        ResultMap r = new ResultMap();
        r.put("msg", HttpStatusEnum.SUCCESS.getMsg());
        r.put("data", data);
        return r;
    }

    public static ResultMap ok(Integer deviceType, Object data, String orderId) {
        ResultMap r = new ResultMap();
        r.put("code", HttpStatusEnum.SUCCESS.getCode());
        r.put("msg", HttpStatusEnum.SUCCESS.getMsg());
        HashMap<String, Object> dataMap = new HashMap<>(2);
        dataMap.put("data", data);
        dataMap.put("deviceType", deviceType);
        dataMap.put("orderId", orderId);
        r.put("data", dataMap);
        return r;
    }

    public static ResultMap ok(Map<String, Object> map) {
        ResultMap r = new ResultMap();
        r.putAll(map);
        return r;
    }

    @Override
    public ResultMap put(String key, Object value) {
        super.put(key, value);
        return this;
    }
}