package com.zanxiang.common.domain;

import com.zanxiang.common.enums.HttpStatusEnum;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.io.Serializable;
import java.util.Objects;

/**
 * @author : lingfeng
 * @time : 2022-06-07
 * @description : http响应消息
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
public class ResultVO<T> implements Serializable {

    /**
     * 序列化
     */
    private static final long serialVersionUID = 1L;

    /**
     * 错误码
     */
    private Integer code;

    /**
     * 错误信息
     */
    private String msg;

    /**
     * 返回内容
     */
    private T data;

    /**
     * 错误消息返回构造
     *
     * @param httpStatusEnum : http响应消息枚举
     */
    public ResultVO(HttpStatusEnum httpStatusEnum) {
        this.code = httpStatusEnum.getCode();
        this.msg = httpStatusEnum.getMsg();
    }

    /**
     * 默认返回成功构造
     *
     * @param data : 返回的内容
     */
    public ResultVO(T data) {
        this.code = HttpStatusEnum.SUCCESS.getCode();
        this.msg = HttpStatusEnum.SUCCESS.getMsg();
        this.data = data;
    }

    /**
     * 消息返回构造
     *
     * @param httpStatusEnum : http响应消息枚举
     * @param data           : 返回的内容
     */
    public ResultVO(HttpStatusEnum httpStatusEnum, T data) {
        this.code = httpStatusEnum.getCode();
        this.msg = httpStatusEnum.getMsg();
        this.data = data;
    }

    /**
     * 自定义消息异常
     *
     * @param code : 错误码
     * @param msg  : 错误消息
     */
    public ResultVO(int code, String msg) {
        this.code = code;
        this.msg = msg;
    }

    /**
     * 判断是否成功
     */
    public boolean isSuccess() {
        return Objects.equals(HttpStatusEnum.SUCCESS.getCode(), this.code);
    }

    public static <T> ResultVO<T> fail(String msg) {
        return new ResultVO<>(HttpStatusEnum.FAIL.getCode(), msg);
    }

    public static <T> ResultVO<T> ok(int code, String msg, T data) {
        return new ResultVO<>(code, msg, data);
    }

    public static <T> ResultVO<T> ok(int code, String msg) {
        return new ResultVO<>(code, msg);
    }

    public static <T> ResultVO<T> ok() {
        return new ResultVO<>(HttpStatusEnum.SUCCESS.getCode(), HttpStatusEnum.SUCCESS.getMsg());
    }

    public static <T> ResultVO<T> ok(T data) {
        return new ResultVO<>(HttpStatusEnum.SUCCESS.getCode(), HttpStatusEnum.SUCCESS.getMsg(), data);
    }
}
