package com.zanxiang.manage.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zanxiang.common.domain.ResultVO;
import com.zanxiang.erp.security.annotation.PreAuthorize;
import com.zanxiang.manage.domain.params.CpAddUpdateParam;
import com.zanxiang.manage.domain.params.CpListParam;
import com.zanxiang.manage.domain.vo.CpChoiceVO;
import com.zanxiang.manage.domain.vo.CpVO;
import com.zanxiang.manage.service.CpService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author : lingfeng
 * @time : 2022-06-22
 * @description : cp管理接口
 */
@Api(tags = {"CP管理接口"})
@RestController
@RequestMapping("/cp")
@Slf4j
public class CpController {

    @Autowired
    private CpService cpService;

    @ApiOperation(value = "cp新增或者更新")
    @PostMapping(value = "/add/or/update")
    @PreAuthorize(permissionKey = "sdk:cp:addOrUpdate")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = Boolean.class)})
    public ResultVO<Boolean> addOrUpdate(@Validated @RequestBody CpAddUpdateParam param) {
        return new ResultVO<>(cpService.addOrUpdate(param));
    }

    @ApiOperation(value = "cp列表查询")
    @PostMapping(value = "/list")
    @PreAuthorize(permissionKey = "sdk:cp:list")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = CpVO.class)})
    public ResultVO<IPage<CpVO>> list(@Validated @RequestBody CpListParam param) {
        return new ResultVO<>(cpService.list(param));
    }

    @ApiOperation(value = "cp删除")
    @DeleteMapping(value = "/delete")
    @PreAuthorize(permissionKey = "sdk:cp:delete")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = Boolean.class)})
    public ResultVO<Boolean> deleteById(@RequestParam Long id) {
        return new ResultVO<>(cpService.deleteById(id));
    }
}
