package com.zanxiang.manage.controller;

import com.zanxiang.common.domain.ResultVO;
import com.zanxiang.erp.security.annotation.PreAuthorize;
import com.zanxiang.manage.domain.params.GameAccountUpdateParam;
import com.zanxiang.manage.domain.vo.GameAccountVO;
import com.zanxiang.manage.service.GameAppletService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @author : lingfeng
 * @time : 2022-07-08
 * @description : 微信小游戏(小程序)
 */
@Api(tags = {"微信小游戏(小程序)"})
@RestController
@RequestMapping("/game/applet")
@Slf4j
public class GameAppletController {

    @Autowired
    private GameAppletService gameAppletService;

    @ApiOperation(value = "获取账号信息配置")
    @GetMapping(value = "/info")
    @PreAuthorize(permissionKey = "sdk:gameApplet:info")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = GameAccountVO.class)})
    public ResultVO<GameAccountVO> getGameAccount(@RequestParam Long id) {
        return new ResultVO<>(gameAppletService.getGameAccount(id));
    }

    @ApiOperation(value = "账号信息配置添加或者更新")
    @PostMapping(value = "/add/or/update")
    @PreAuthorize(permissionKey = "sdk:gameApplet:addOrUpdate")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = Boolean.class)})
    public ResultVO<Boolean> gameAccountAddOrUpdate(@Validated @RequestBody GameAccountUpdateParam param) {
        return new ResultVO<>(gameAppletService.gameAccountAddOrUpdate(param));
    }

    @ApiOperation(value = "获取小游戏二维码")
    @GetMapping(value = "/qr/code")
    @PreAuthorize(permissionKey = "sdk:gameApplet:qrCode")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = String.class)})
    public ResultVO<String> getQrCode(@RequestParam Long gameId) {
        return new ResultVO<>(gameAppletService.getQrCod(gameId));
    }
}
