package com.zanxiang.manage.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zanxiang.common.domain.ResultVO;
import com.zanxiang.erp.security.annotation.PreAuthorize;
import com.zanxiang.manage.domain.params.GameCategoryAddUpdateParam;
import com.zanxiang.manage.domain.params.GameCategoryListParam;
import com.zanxiang.manage.domain.vo.GameCategoryParentVO;
import com.zanxiang.manage.domain.vo.GameCategoryVO;
import com.zanxiang.manage.service.GameCategoryService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author : lingfeng
 * @time : 2022-06-23
 * @description : 游戏分类标签
 */
@Api(tags = {"游戏分类标签管理接口"})
@RestController
@RequestMapping("/game/category")
@Slf4j
public class GameCategoryController {

    @Autowired
    private GameCategoryService gameCategoryService;

    @ApiOperation(value = "查询所游戏分类或父标签列表")
    @GetMapping(value = "/parent/list")
    @PreAuthorize(permissionKey = "sdk:gameCategory:parentList")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = GameCategoryParentVO.class, responseContainer = "list")})
    public ResultVO<List<GameCategoryParentVO>> getGameCategoryParent(@RequestParam Integer type) {
        return ResultVO.ok(gameCategoryService.getGameCategoryParent(type));
    }

    @ApiOperation(value = "查询游戏标签列表")
    @PostMapping(value = "/list")
    @PreAuthorize(permissionKey = "sdk:gameCategory:list")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = GameCategoryVO.class)})
    public ResultVO<IPage<GameCategoryVO>> list(@Validated @RequestBody GameCategoryListParam param) {
        return ResultVO.ok(gameCategoryService.list(param));
    }

    @ApiOperation(value = "游戏标签分类添加或修改")
    @PostMapping(value = "/add/or/update")
    @PreAuthorize(permissionKey = "sdk:gameCategory:addOrUpdate")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = Boolean.class)})
    public ResultVO<Boolean> addOrUpdate(@Validated @RequestBody GameCategoryAddUpdateParam param) {
        return ResultVO.ok(gameCategoryService.addOrUpdate(param));
    }

    @ApiOperation(value = "游戏标签分类删除")
    @DeleteMapping(value = "/delete/by/id")
    @PreAuthorize(permissionKey = "sdk:gameCategory:deleteById")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = Boolean.class)})
    public ResultVO<Boolean> deleteById(@RequestParam Long id) {
        return ResultVO.ok(gameCategoryService.deleteById(id));
    }
}
