package com.zanxiang.manage.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zanxiang.common.domain.ResultVO;
import com.zanxiang.erp.security.annotation.PreAuthorize;
import com.zanxiang.manage.domain.params.GameAnalogPayParam;
import com.zanxiang.manage.domain.params.GamePayWayListParam;
import com.zanxiang.manage.domain.params.GamePayWayParam;
import com.zanxiang.manage.domain.vo.AllGamePayWayByGameIdVO;
import com.zanxiang.manage.domain.vo.GameAnalogPayVO;
import com.zanxiang.manage.domain.vo.GamePayWayListVO;
import com.zanxiang.manage.service.GamePayWayService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * 游戏支付配置表 前端控制器
 *
 * @author xufeng
 * @date 2022-06-30 14:22
 */
@Api(tags = "游戏支付配置管理")
@RestController
@RequestMapping("/game-pay-way")
public class GamePayWayController {

    @Autowired
    private GamePayWayService gamePayWayService;

    @ApiOperation(value = "新增/编辑/删除")
    @PostMapping(value = "/save")
    @PreAuthorize(permissionKey = "sdk:gamePayWay:save")
    public ResultVO<Boolean> save(@Validated @RequestBody GamePayWayParam param) {
        return ResultVO.ok(gamePayWayService.saveOrUpdate(param));
    }

    @ApiOperation(value = "列表")
    @PostMapping(value = "/list")
    @PreAuthorize(permissionKey = "sdk:gamePayWay:list")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = GamePayWayListVO.class, responseContainer = "list")})
    public ResultVO<IPage<GamePayWayListVO>> list(@Validated @RequestBody GamePayWayListParam param) {
        return ResultVO.ok(gamePayWayService.list(param));
    }

    @ApiOperation(value = "游戏详情支付配置展示")
    @GetMapping(value = "/allGamePayWayByGameId")
    @PreAuthorize(permissionKey = "sdk:gamePayWay:allGamePayWayByGameId")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = AllGamePayWayByGameIdVO.class)})
    public ResultVO<AllGamePayWayByGameIdVO> getInfo(@RequestParam Long gameId) {
        return new ResultVO<>(gamePayWayService.getAllGamePayWayByGameId(gameId));
    }

    @ApiOperation(value = "获取游戏虚拟支付配置")
    @GetMapping(value = "/analog/pay/info")
    @PreAuthorize(permissionKey = "sdk:gamePayWay:analogPayInfo")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = GameAnalogPayVO.class)})
    public ResultVO<GameAnalogPayVO> getGameAnalogPay(@RequestParam Long id) {
        return new ResultVO<>(gamePayWayService.getGameAnalogPay(id));
    }

    @ApiOperation(value = "游戏虚拟支付配置添加或者更新")
    @PostMapping(value = "/analog/pay/add/or/update")
    @PreAuthorize(permissionKey = "sdk:gamePayWay:analogPayAddOrUpdate")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = Boolean.class)})
    public ResultVO<Boolean> gameAnalogPayAddOrUpdate(@Validated @RequestBody GameAnalogPayParam param) {
        return new ResultVO<>(gamePayWayService.gameAnalogPayAddOrUpdate(param));
    }
}
