package com.zanxiang.manage.controller;

import com.zanxiang.common.domain.ResultVO;
import com.zanxiang.common.enums.FileTypeEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.erp.security.annotation.PreAuthorize;
import com.zanxiang.module.oss.pojo.OssUploadCallback;
import com.zanxiang.module.oss.pojo.dto.OssUploadDTO;
import com.zanxiang.module.oss.service.IOssService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * @author : lingfeng
 * @time : 2022-07-12
 * @description : oss相关接口
 */
@Api(tags = {"oss相关接口管理"})
@RestController
@RequestMapping("/oss")
@Slf4j
public class OssController {

    @Autowired
    private IOssService ossService;

    @ApiOperation(value = "文件上传oss")
    @GetMapping(value = "/form/upload")
    @PreAuthorize(permissionKey = "sdk:oss:formUpload")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = Map.class)})
    public ResultVO<Map<String, String>> formUpload(@RequestParam String fileType, @RequestParam String type) {
        FileTypeEnum fileTypeEnum = FileTypeEnum.getByFileType(fileType);
        if (fileTypeEnum == null) {
            throw new BaseException("错误的文件类型");
        }
        return new ResultVO<>(ossService.formUpload(fileTypeEnum.getFilePath(), type));
    }

    @ApiOperation(value = "oss回传接口")
    @PostMapping("/callback")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = OssUploadCallback.class)})
    public ResultVO<OssUploadCallback> ossCallback(@RequestBody OssUploadDTO dto) {
        return ResultVO.ok(ossService.callback(dto));
    }

}
