package com.zanxiang.manage.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zanxiang.common.domain.ResultVO;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.erp.security.annotation.PreAuthorize;
import com.zanxiang.manage.domain.bo.PayApplicationBO;
import com.zanxiang.manage.domain.params.PayApplicationListNoPageParam;
import com.zanxiang.manage.domain.params.PayApplicationListParam;
import com.zanxiang.manage.domain.params.PayApplicationParam;
import com.zanxiang.manage.domain.vo.PayApplicationVO;
import com.zanxiang.manage.service.PayApplicationService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 游戏应用
 *
 * @author xufeng
 * @date 2022-06-20 15:16
 */
@Api(tags = "应用管理")
@RestController
@RequestMapping("/pay-application")
public class PayApplicationController {

    @Autowired
    private PayApplicationService payApplicationService;

    @ApiOperation(value = "新增/编辑/删除")
    @PostMapping(value = "/save")
    @PreAuthorize(permissionKey = "sdk:payApplication:save")
    public ResultVO<Boolean> save(@Validated @RequestBody PayApplicationParam param) {
        Boolean result = payApplicationService.saveOrUpdate(BeanUtils.copy(param, PayApplicationBO.class));
        return ResultVO.ok(result);
    }

    @ApiOperation(value = "列表")
    @PostMapping(value = "/list")
    @PreAuthorize(permissionKey = "sdk:payApplication:list")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = PayApplicationVO.class, responseContainer = "list")})
    public ResultVO<IPage<PayApplicationVO>> list(@Validated @RequestBody PayApplicationListParam param) {
        return ResultVO.ok(payApplicationService.list(param));
    }

    @ApiOperation(value = "列表(无分页)")
    @PostMapping(value = "/listNoPage")
    @PreAuthorize(permissionKey = "sdk:payApplication:listNoPage")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = PayApplicationVO.class, responseContainer = "list")})
    public ResultVO<List<PayApplicationVO>> listNoPage(@Validated @RequestBody PayApplicationListNoPageParam param) {
        return ResultVO.ok(payApplicationService.listNoPage(param));
    }

    @ApiOperation(value = "获取小程序二维码")
    @GetMapping(value = "/qr/code")
    @PreAuthorize(permissionKey = "sdk:payApplication:qrCode")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = String.class)})
    public ResultVO<String> getQrCode(@RequestParam Long id) {
        return new ResultVO<>(payApplicationService.getQrCode(id));
    }
}
