package com.zanxiang.manage.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zanxiang.common.domain.ResultVO;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.erp.security.annotation.PreAuthorize;
import com.zanxiang.manage.domain.bo.PayMerchantBO;
import com.zanxiang.manage.domain.params.PayMerchantListNoPageParam;
import com.zanxiang.manage.domain.params.PayMerchantListParam;
import com.zanxiang.manage.domain.params.PayMerchantParam;
import com.zanxiang.manage.domain.vo.PayMerchantListNoPageVO;
import com.zanxiang.manage.domain.vo.PayMerchantListVO;
import com.zanxiang.manage.service.PayMerchantService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 游戏支付商户配置表 前端控制器
 *
 * @author xufeng
 * @date 2022-06-16 19:04
 */
@Api(tags = "商户号管理")
@RestController
@RequestMapping("/pay-merchant")
public class PayMerchantController {

    @Autowired
    private PayMerchantService payMerchantService;

    @ApiOperation(value = "商户号-新增/编辑/删除(payWayId/merchantNo/merchantName 必传)")
    @PostMapping(value = "/save")
    @PreAuthorize(permissionKey = "sdk:payMerchant:save")
    public ResultVO<Boolean> save(@Validated @RequestBody PayMerchantParam param) {
        Boolean result = payMerchantService.saveOrUpdate(BeanUtils.copy(param, PayMerchantBO.class));
        if (result) {
            return ResultVO.ok(true);
        }
        return ResultVO.ok(false);
    }

    @ApiOperation(value = "商户号-列表")
    @PostMapping(value = "/list")
    @PreAuthorize(permissionKey = "sdk:payMerchant:list")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = PayMerchantListVO.class, responseContainer = "list")})
    public ResultVO<IPage<PayMerchantListVO>> list(@Validated @RequestBody PayMerchantListParam param) {
        IPage<PayMerchantListVO> result = payMerchantService.list(param);
        return ResultVO.ok(result);
    }

    @ApiOperation(value = "列表(无分页)")
    @PostMapping(value = "/listNoPage")
    @PreAuthorize(permissionKey = "sdk:payMerchant:listNoPage")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = PayMerchantListNoPageVO.class, responseContainer = "list")})
    public ResultVO<List<PayMerchantListNoPageVO>> listNoPage(@Validated @RequestBody PayMerchantListNoPageParam param) {
        return ResultVO.ok(payMerchantService.listNoPage(param));
    }

}
