package com.zanxiang.manage.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zanxiang.common.domain.ResultVO;
import com.zanxiang.erp.security.annotation.PreAuthorize;
import com.zanxiang.manage.domain.params.UserListParam;
import com.zanxiang.manage.domain.params.UserLoginLogParam;
import com.zanxiang.manage.domain.params.UserOrderListParam;
import com.zanxiang.manage.domain.params.UserUpdateParam;
import com.zanxiang.manage.domain.vo.*;
import com.zanxiang.manage.service.OrderService;
import com.zanxiang.manage.service.UserAddressService;
import com.zanxiang.manage.service.UserLoginLogService;
import com.zanxiang.manage.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author : lingfeng
 * @time : 2022-06-29
 * @description : 玩家管理接口
 */
@Api(tags = {"玩家管理接口"})
@RestController
@RequestMapping("/user")
@Slf4j
public class UserController {

    @Autowired
    private UserService userService;

    @Autowired
    private UserAddressService userAddressService;

    @Autowired
    private UserLoginLogService userLoginLogService;

    @Autowired
    private OrderService orderService;

    @ApiOperation(value = "玩家列表")
    @PostMapping(value = "/list")
    @PreAuthorize(permissionKey = "sdk:user:list")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = UserListVO.class)})
    public ResultVO<IPage<UserListVO>> list(@Validated @RequestBody UserListParam param) {
        return ResultVO.ok(userService.list(param));
    }

    @ApiOperation(value = "玩家信息编辑")
    @PostMapping(value = "/update")
    @PreAuthorize(permissionKey = "sdk:user:update")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = Boolean.class)})
    public ResultVO<Boolean> userUpdate(@Validated @RequestBody UserUpdateParam param) {
        return ResultVO.ok(userService.userUpdate(param));
    }

    @ApiOperation(value = "玩家详细信息")
    @GetMapping(value = "/info")
    @PreAuthorize(permissionKey = "sdk:user:info")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = UserVO.class)})
    public ResultVO<UserVO> getUserInfo(@RequestParam Long id) {
        return ResultVO.ok(userService.getUserInfo(id));
    }

    @ApiOperation(value = "玩家登录记录列表")
    @PostMapping(value = "/login/list")
    @PreAuthorize(permissionKey = "sdk:user:loginList")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = UserRoleLoginVO.class)})
    public ResultVO<IPage<UserRoleLoginVO>> loginList(@RequestBody UserLoginLogParam param) {
        return ResultVO.ok(userLoginLogService.loginList(param));
    }

    @ApiOperation(value = "玩家下单记录列表")
    @PostMapping(value = "/order/list")
    @PreAuthorize(permissionKey = "sdk:user:orderList")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = UserOrderListVO.class)})
    public ResultVO<IPage<UserOrderListVO>> orderList(@Validated @RequestBody UserOrderListParam param) {
        return ResultVO.ok(orderService.orderList(param));
    }

    @ApiOperation(value = "玩家收货地址列表")
    @GetMapping(value = "/address/list")
    @PreAuthorize(permissionKey = "sdk:user:addressList")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = UserAddressVO.class, responseContainer = "list")})
    public ResultVO<List<UserAddressVO>> list(@RequestParam Long userId) {
        return ResultVO.ok(userAddressService.list(userId));
    }
}
