package com.zanxiang.manage.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.zanxiang.common.domain.ResultVO;
import com.zanxiang.erp.security.annotation.PreAuthorize;
import com.zanxiang.manage.domain.params.GameUserRoleListParam;
import com.zanxiang.manage.domain.vo.GameUserRoleListVO;
import com.zanxiang.manage.domain.vo.GameUserRoleVO;
import com.zanxiang.manage.service.GameUserRoleService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author : lingfeng
 * @time : 2022-11-03
 * @description : 玩家角色
 */
@Api(tags = {"玩家角色"})
@RestController
@RequestMapping("/user")
@Slf4j
public class UserRoleController {

    @Autowired
    private GameUserRoleService gameUserRoleService;

    @ApiOperation(value = "用户游戏角色")
    @GetMapping(value = "/game/role/list")
    @PreAuthorize(permissionKey = "sdk:user:gameRoleList")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = GameUserRoleVO.class, responseContainer = "list")})
    public ResultVO<List<GameUserRoleVO>> roleList(@RequestParam Long userId) {
        return ResultVO.ok(gameUserRoleService.roleList(userId));
    }

    @ApiOperation(value = "玩家游戏角色列表")
    @PostMapping(value = "/role/list")
    @PreAuthorize(permissionKey = "sdk:user:roleList")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = GameUserRoleListVO.class)})
    public ResultVO<IPage<GameUserRoleListVO>> roleList(@Validated @RequestBody GameUserRoleListParam param) {
        return ResultVO.ok(gameUserRoleService.list(param));
    }
}
