package com.zanxiang.manage.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.zanxiang.manage.domain.dto.GameCategoryDTO;
import com.zanxiang.manage.domain.params.GameCategoryAddUpdateParam;
import com.zanxiang.manage.domain.params.GameCategoryListParam;
import com.zanxiang.manage.domain.vo.GameCategoryParentVO;
import com.zanxiang.manage.domain.vo.GameCategoryVO;
import com.zanxiang.mybatis.entity.GameCategory;

import java.util.List;
import java.util.Map;

/**
 * @author : lingfeng
 * @time : 2022-06-23
 * @description : 游戏分类标签管理
 */
public interface GameCategoryService extends IService<GameCategory> {

    /**
     * 根据主键id查询分类以及子分类
     *
     * @param idList : 主键id列表
     * @param type   : 类型
     * @return : 返回游戏分类信息
     */
    List<GameCategoryVO> listByIdList(List<Long> idList, Integer type);

    /**
     * 根据主键id查询
     *
     * @param id : 主键id
     * @return : 返回游戏分类信息
     */
    GameCategoryVO getById(Long id);

    /**
     * 查询所有父标签列表
     *
     * @param type : 标签分类
     * @return : 返回所有父游戏标签
     */
    List<GameCategoryParentVO> getGameCategoryParent(Integer type);

    /**
     * 查询游戏分类和标签
     *
     * @return : 返回所有游戏分类和标签
     */
    Map<Long, GameCategoryDTO> gameCategoryMap();

    /**
     * 查询游戏标签列表
     *
     * @param param : 游戏标签列表查询呢参数
     * @return : 返回分页信息
     */
    IPage<GameCategoryVO> list(GameCategoryListParam param);

    /**
     * 游戏标签分类添加或修改
     *
     * @param param : 参数
     * @return : 返回结果
     */
    Boolean addOrUpdate(GameCategoryAddUpdateParam param);

    /**
     * 根据主键id删除
     *
     * @param id : 主键id
     * @return : 删除结果
     */
    Boolean deleteById(Long id);
}
