package com.zanxiang.manage.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.zanxiang.manage.domain.params.GameAnalogPayParam;
import com.zanxiang.manage.domain.params.GamePayWayListParam;
import com.zanxiang.manage.domain.params.GamePayWayParam;
import com.zanxiang.manage.domain.vo.AllGamePayWayByGameIdVO;
import com.zanxiang.manage.domain.vo.GameAnalogPayVO;
import com.zanxiang.manage.domain.vo.GamePayWayListVO;
import com.zanxiang.manage.domain.vo.GamePayWayVO;
import com.zanxiang.mybatis.entity.GamePayWay;

import java.util.List;

/**
 * 游戏支付配置表 服务类接口
 *
 * @author xufeng
 * @date 2022-06-30 14:22
 */
public interface GamePayWayService extends IService<GamePayWay> {

    /**
     * 根据游戏id查询
     *
     * @param gameId
     * @return
     */
    List<GamePayWayVO> getByGameId(Long gameId);

    /**
     * 根据游戏id查询(状态yes)
     *
     * @param gameId
     * @return
     */
    List<GamePayWayVO> getByGameIdForStatusYes(Long gameId);

    /**
     * 游戏配置更新/保存/删除
     *
     * @param bo
     * @return
     */
    Boolean saveOrUpdate(GamePayWayParam bo);

    /**
     * 带分页列表
     *
     * @param param
     * @return
     */
    IPage<GamePayWayListVO> list(GamePayWayListParam param);

    /**
     * 根据游戏id获取虚拟支付配置
     *
     * @param id : 游戏id
     * @return : 返回游戏虚拟支付配置
     */
    GameAnalogPayVO getGameAnalogPay(Long id);

    /**
     * 虚拟支付更新或者新增
     *
     * @param payParam : 新增或者更新参数
     * @return : 返回新增或者更新的结果
     */
    Boolean gameAnalogPayAddOrUpdate(GameAnalogPayParam payParam);

    /**
     * 获取游戏所有支付方式
     *
     * @param gameId 游戏id
     * @return
     */
    AllGamePayWayByGameIdVO getAllGamePayWayByGameId(Long gameId);
}
