package com.zanxiang.manage.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.zanxiang.manage.domain.dto.GameDTO;
import com.zanxiang.manage.domain.params.*;
import com.zanxiang.manage.domain.vo.*;
import com.zanxiang.mybatis.entity.Game;

import java.util.List;
import java.util.Map;

/**
 * @author : lingfeng
 * @time : 2022-06-24
 * @description : 游戏逻辑
 */
public interface GameService extends IService<Game> {

    /**
     * 获取游戏对接参数
     *
     * @param id : 游戏id
     * @return : 信息展示
     */
    GameDockVO getGameDock(Long id);

    /**
     * 参数对接更新
     *
     * @param param : 对接参数
     * @return : 返回更新结果
     */
    Boolean gameDockUpdate(GameDockParam param);

    /**
     * 获取游戏关联信息
     *
     * @param id : 游戏id
     * @return : 返回游戏关联信息
     */
    GameRelationVO getGameRelation(Long id);

    /**
     * 导量游戏更新
     *
     * @param param : 关联游戏更新参数
     * @return : 返回更新结果
     */
    Boolean guideGameUpdateUpdate(GuideGameUpdateParam param);

    /**
     * 关联游戏更新
     *
     * @param param : 关联游戏更新参数
     * @return : 返回更新结果
     */
    Boolean relationGameUpdate(RelationGameUpdateParam param);

    /**
     * 获取游戏基本信息
     *
     * @param gameId : 游戏id
     * @return : 返回游戏信息
     */
    GameInfoVO getGameInfo(Long gameId);

    /**
     * 游戏基本信息更新
     *
     * @param param : 游戏更新参数
     * @return : 返回更新结果
     */
    Boolean updateGameInfo(GameUpdateParam param);

    /**
     * 新增游戏
     *
     * @param param : 游戏新增参数
     * @return : 返回添加结果
     */
    Boolean gameAdd(GameAddParam param);

    /**
     * 查询游戏列表
     *
     * @param param : 游戏列表查询参数
     * @return : 返回单页游戏游戏列表数据
     */
    IPage<GameListVO> gameList(GameListParam param);

    /**
     * 游戏选择列表查询
     *
     * @return : 返回所有游戏信息
     */
    List<GameChoiceVO> choiceList();

    /**
     * 根据id查询游戏信息
     *
     * @param id : 游戏id
     * @return : 返回游戏信息
     */
    GameDTO getById(Long id);

    /**
     * 游戏相关条件查询游戏信息
     *
     * @param cpId           : cp唯一键
     * @param gameId         : 游戏id
     * @param gameCategoryId : 分类id
     * @return : 返回游戏信息列表
     */
    Map<Long, GameDTO> gameCondition(Long cpId, Long gameId, Long gameCategoryId);

    /**
     * 根据主键id删除
     *
     * @param id : 主键id
     * @return : 返回删除结果
     */
    Boolean deleteById(Long id);

    /**
     * 根据主键id删除
     *
     * @param id : 主键id
     * @return : 返回删除结果
     */
    Boolean statusUpdate(Long id, Integer status);
}
