package com.zanxiang.manage.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.manage.domain.dto.GameDTO;
import com.zanxiang.manage.domain.params.GameAccountUpdateParam;
import com.zanxiang.manage.domain.vo.GameAccountVO;
import com.zanxiang.manage.service.GameAppletService;
import com.zanxiang.manage.service.GameService;
import com.zanxiang.manage.service.api.MiniAppletApiService;
import com.zanxiang.mybatis.entity.GameApplet;
import com.zanxiang.mybatis.mapper.GameAppletMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;

/**
 * @author : lingfeng
 * @time : 2022-07-08
 * @description : 微信小游戏(小程序)
 */
@Slf4j
@Service
public class GameAppletServiceImpl extends ServiceImpl<GameAppletMapper, GameApplet> implements GameAppletService {

    @Autowired
    private MiniAppletApiService miniAppletApiService;

    @Autowired
    private GameService gameService;

    /**
     * 获取账号信息配置
     *
     * @param id : 游戏id
     * @return : 账号配置信息
     */
    @Override
    public GameAccountVO getGameAccount(Long id) {
        GameApplet gameApplet = super.getOne(new LambdaQueryWrapper<GameApplet>()
                .eq(GameApplet::getGameId, id));
        if (gameApplet == null) {
            GameAccountVO vo = new GameAccountVO();
            vo.setGameId(id);
            return vo;
        }
        return BeanUtils.copy(gameApplet, GameAccountVO.class);
    }

    /**
     * 账号配置信息更新或者添加
     *
     * @param param : 更新参数
     * @return : 账号配置信息
     */
    @Override
    public Boolean gameAccountAddOrUpdate(GameAccountUpdateParam param) {
        GameApplet gameApplet;
        if (param.getId() == null) {
            gameApplet = new GameApplet();
            gameApplet.setCreateTime(LocalDateTime.now());
            gameApplet.setGameId(param.getGameId());
        } else {
            gameApplet = super.getById(param.getId());
        }
        gameApplet.setAppId(param.getAppId());
        gameApplet.setGhId(param.getGhId());
        gameApplet.setAccount(param.getAccount());
        gameApplet.setPassword(param.getPassword());
        gameApplet.setAdminVx(param.getAdminVx());
        gameApplet.setCompany(param.getCompany());
        gameApplet.setAppSecret(param.getAppSecret());
        gameApplet.setUpdateTime(LocalDateTime.now());
        return super.saveOrUpdate(gameApplet);
    }

    /**
     * 获取游戏小程序二维码
     *
     * @param gameId : 游戏id
     * @return : 返回游戏二维码
     */
    @Override
    public String getQrCod(Long gameId) {
        GameDTO gameDTO = gameService.getById(gameId);
        if (gameDTO == null) {
            throw new BaseException("游戏不存在");
        }
        GameApplet gameApplet = super.getOne(new LambdaQueryWrapper<GameApplet>().eq(GameApplet::getGameId, gameId));
        if (gameApplet == null || Strings.isBlank(gameApplet.getAppId()) || Strings.isBlank(gameApplet.getAppSecret())) {
            throw new BaseException("该游戏未设置小程序信息");
        }
        return miniAppletApiService.getQrCode(gameApplet.getAppId(), gameApplet.getAppSecret(), gameDTO.getName());
    }
}
