package com.zanxiang.manage.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.enums.DeleteEnum;
import com.zanxiang.common.enums.PayWayEnum;
import com.zanxiang.common.enums.StatusEnum;
import com.zanxiang.common.exception.ParamNullException;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.manage.domain.mapper.GamePayWayListMapper;
import com.zanxiang.manage.domain.params.GameAnalogPayParam;
import com.zanxiang.manage.domain.params.GamePayWayListParam;
import com.zanxiang.manage.domain.params.GamePayWayParam;
import com.zanxiang.manage.domain.vo.*;
import com.zanxiang.manage.service.*;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.mybatis.entity.*;
import com.zanxiang.mybatis.mapper.GamePayWayMapper;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * 游戏支付配置表 服务实现类
 *
 * @author xufeng
 * @date 2022-06-30 14:22
 */
@Service
public class GamePayWayServiceImpl extends ServiceImpl<GamePayWayMapper, GamePayWay> implements GamePayWayService {

    @Autowired
    private GamePayWayListMapper gamePayWayListMapper;

    @Autowired
    private PayMerchantService payMerchantService;

    @Autowired
    private PayBoxService payBoxService;

    @Autowired
    private PayApplicationService payApplicationService;

    @Autowired
    private GameStrategyService gameStrategyService;

    @Override
    public List<GamePayWayVO> getByGameId(Long gameId) {
        List<GamePayWay> gamePayWayList = super.list(new LambdaQueryWrapper<GamePayWay>()
                .eq(GamePayWay::getGameId, gameId)
                .select(GamePayWay::getId, GamePayWay::getPayWayName));
        return BeanUtils.copyList(gamePayWayList, GamePayWayVO.class);
    }


    @Override
    public List<GamePayWayVO> getByGameIdForStatusYes(Long gameId) {
        List<GamePayWay> gamePayWayList = super.list(new LambdaQueryWrapper<GamePayWay>()
                .eq(GamePayWay::getGameId, gameId)
                .eq(GamePayWay::getStatus, StatusEnum.YES.getCode())
                .select(GamePayWay::getId, GamePayWay::getPayWayName));
        return BeanUtils.copyList(gamePayWayList, GamePayWayVO.class);
    }

    @Override
    public Boolean saveOrUpdate(GamePayWayParam bo) {
        if (bo.getIsDelete() != null && bo.getIsDelete() == DeleteEnum.YES.getCode()) {
            if (bo.getId() == null) {
                throw new ParamNullException("Id");
            }
            return removeById(bo.getId());
        }
        return saveOrUpdate(BeanUtils.copy(bo, GamePayWay.class));
    }

    @Override
    public IPage<GamePayWayListVO> list(GamePayWayListParam param) {

        Page<GamePayWay> page = new Page<>(param.getPageNum(), param.getPageSize());
        String payWayId = param.getPayWayId() == null ? null : String.valueOf(param.getPayWayId());
        String status = param.getStatus() == null ? null : String.valueOf(param.getStatus());
        return gamePayWayListMapper.List(page, param.getGameName(), payWayId, status);
    }

    /**
     * 根据游戏id获取虚拟支付配置
     *
     * @param id : 游戏id
     * @return : 返回游戏虚拟支付配置
     */
    @Override
    public GameAnalogPayVO getGameAnalogPay(Long id) {
        GamePayWay gamePayWay = super.getOne(new LambdaQueryWrapper<GamePayWay>()
                .eq(GamePayWay::getPayWayId, PayWayEnum.MIPAY.getNum())
                .eq(GamePayWay::getGameId, id));
        if (gamePayWay == null || Strings.isBlank(gamePayWay.getPayConfig())) {
            return GameAnalogPayVO.builder().gameId(id).build();
        }
        return GameAnalogPayVO.builder()
                .id(gamePayWay.getId())
                .gameId(id)
                .payConfig(JsonUtil.toObj(gamePayWay.getPayConfig(), GameAnalogPayVO.PayConfigBean.class))
                .build();
    }

    /**
     * 虚拟支付更新或者新增
     *
     * @param payParam : 新增或者更新参数
     * @return : 返回新增或者更新的结果
     */
    @Override
    public Boolean gameAnalogPayAddOrUpdate(GameAnalogPayParam payParam) {
        GamePayWay gamePayWay = super.getOne(new LambdaQueryWrapper<GamePayWay>()
                .eq(GamePayWay::getPayWayId, PayWayEnum.MIPAY.getNum())
                .eq(GamePayWay::getGameId, payParam.getGameId()));
        if (gamePayWay == null) {
            gamePayWay = GamePayWay.builder()
                    .gameId(payParam.getGameId())
                    .payWayId(PayWayEnum.MIPAY.getNum())
                    .payWayCode(PayWayEnum.MIPAY.getCode())
                    .payWayName(PayWayEnum.MIPAY.getName())
                    .payConfig(JsonUtil.toString(payParam.getPayConfig()))
                    .createTime(LocalDateTime.now())
                    .updateTime(LocalDateTime.now())
                    .build();
        } else {
            gamePayWay.setPayConfig(JsonUtil.toString(payParam.getPayConfig()));
            gamePayWay.setUpdateTime(LocalDateTime.now());
        }
        return super.saveOrUpdate(gamePayWay);
    }

    @Override
    public AllGamePayWayByGameIdVO getAllGamePayWayByGameId(Long gameId) {
        List<GamePayWay> gamePayWayList = super.list(new LambdaQueryWrapper<GamePayWay>()
                .eq(GamePayWay::getGameId, gameId)
                .eq(GamePayWay::getStatus, StatusEnum.YES.getCode())
        );
        if (CollectionUtils.isEmpty(gamePayWayList)) {
            return null;
        }
        AllGamePayWayByGameIdVO data = new AllGamePayWayByGameIdVO();
        gamePayWayList.stream().forEach(t -> {
            //支付宝
            if (PayWayEnum.ALIPAY.getNum().equals(t.getPayWayId())) {
                if (Objects.isNull(t.getPayMerchantId()) || t.getPayMerchantId() == 0) {
                    //商户号为空则跳过
                    return;
                }
                PayMerchant payMerchantInfo = payMerchantService.getById(t.getPayMerchantId());
                if (Objects.isNull(payMerchantInfo)) {
                    //商户号内容为空
                    return;
                }
                GameAlipayPayVO alipayPayVO = new GameAlipayPayVO();
                alipayPayVO.setMerchantNo(payMerchantInfo.getMerchantNo());
                alipayPayVO.setMerchantName(payMerchantInfo.getMerchantName());
                alipayPayVO.setLoginName(payMerchantInfo.getLoginName());
                alipayPayVO.setManagePerson(payMerchantInfo.getManagePerson());
                data.setAlipayPayVO(alipayPayVO);
            }
            //微信
            if (PayWayEnum.WXPAY.getNum().equals(t.getPayWayId())) {

                if (Objects.isNull(t.getPayBoxId()) || t.getPayBoxId() == 0) {
                    //商户号为空则跳过
                    return;
                }
                PayBox payBoxInfo = payBoxService.getById(t.getPayBoxId());
                if (Objects.isNull(payBoxInfo) || Objects.isNull(payBoxInfo.getPayApplicationId()) ||
                        payBoxInfo.getPayApplicationId() <= 0) {
                    return;
                }
                PayApplication payApplicationInfo = payApplicationService.getById(payBoxInfo.getPayApplicationId());
                if (Objects.isNull(payApplicationInfo)) {
                    return;
                }
                GameWxpayPayVO gameWxpayPayVO = new GameWxpayPayVO();
                gameWxpayPayVO.setBoxName(payBoxInfo.getName());
                gameWxpayPayVO.setApplicationName(payApplicationInfo.getName());
                gameWxpayPayVO.setAppId(payApplicationInfo.getAppId());
                gameWxpayPayVO.setPrimevalId(payApplicationInfo.getPrimevalId());
                gameWxpayPayVO.setCompany(payApplicationInfo.getCompany());
                gameWxpayPayVO.setManagePerson(payApplicationInfo.getManagePerson());

                //获取商户号列表
                List<PayMerchant> payMerchantList = payMerchantService.list();
                Map<String, PayMerchant> payMerchantData = payMerchantList.stream().collect(Collectors.toMap(PayMerchant::getMerchantNo, PayMerchant -> PayMerchant));
                if (StringUtils.isNotEmpty(payBoxInfo.getPayMerchantId())) {
                    String merchantDetail = payBoxService.getMerchantDetail(payMerchantData, payBoxInfo.getPayMerchantId());
                    gameWxpayPayVO.setPayMerchantDetail(merchantDetail);
                }
                data.setWxpayPayVO(gameWxpayPayVO);
            }
            //米大师虚拟支付
            if (PayWayEnum.MIPAY.getNum().equals(t.getPayWayId())) {
                GameAnalogPayVO.PayConfigBean payConfigBean = JsonUtil.toObj(t.getPayConfig(), GameAnalogPayVO.PayConfigBean.class);
                data.setAnalogPayVO(payConfigBean);
            }
            //支付策略配置
            GameStrategy gameStrategy = gameStrategyService.getOne(new LambdaQueryWrapper<GameStrategy>()
                    .eq(GameStrategy::getGameId, gameId)
                    .eq(GameStrategy::getStatus, StatusEnum.YES.getCode()).last("limit 1")
            );
            if (!Objects.isNull(gameStrategy)) {
                GameStrategyInfoVO gameStrategyInfo = BeanUtils.copy(gameStrategy, GameStrategyInfoVO.class);
                if (StringUtils.isNotEmpty(gameStrategy.getConfig())) {
                    gameStrategyInfo.setConfig(BeanUtils.copy(gameStrategy.getConfig(), GameStrategyOrderVO.class));
                }
                data.setGameStrategyVO(gameStrategyInfo);
            }
        });
        return data;
    }
}
