package com.zanxiang.manage.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.enums.GameCategoryEnum;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.manage.domain.dto.GameCategoryDTO;
import com.zanxiang.manage.domain.dto.GameDTO;
import com.zanxiang.manage.domain.params.*;
import com.zanxiang.manage.domain.vo.*;
import com.zanxiang.manage.service.*;
import com.zanxiang.mybatis.entity.Cp;
import com.zanxiang.mybatis.entity.Game;
import com.zanxiang.mybatis.entity.GameApplet;
import com.zanxiang.mybatis.entity.GameCategory;
import com.zanxiang.mybatis.mapper.GameMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.time.LocalDateTime;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2022-06-24
 * @description : 游戏逻辑
 */
@Slf4j
@Service
public class GameServiceImpl extends ServiceImpl<GameMapper, Game> implements GameService {

    @Autowired
    private GamePayWayService gamePayWayService;

    @Autowired
    private GameStrategyService gameStrategyService;

    @Autowired
    private GameCategoryService gameCategoryService;

    @Autowired
    private CpService cpService;

    @Autowired
    private GameAppletService gameAppletService;

    /**
     * 获取游戏对接参数
     *
     * @param id : 游戏id
     * @return : 信息展示
     */
    @Override
    public GameDockVO getGameDock(Long id) {
        Game game = super.getById(id);
        GameDockVO gameDockVO = BeanUtils.copy(game, GameDockVO.class);
        if (gameDockVO == null) {
            return null;
        }
        Cp cp = cpService.getById(gameDockVO.getCpId());
        if (cp != null) {
            gameDockVO.setCpName(cp.getCpName());
        }
        GameApplet gameApplet = gameAppletService.getOne(new LambdaQueryWrapper<GameApplet>().eq(GameApplet::getGameId, gameDockVO.getId()));
        if (gameApplet != null) {
            gameDockVO.setAppId(gameApplet.getAppId());
            gameDockVO.setAppSecret(gameApplet.getAppSecret());
        }
        return gameDockVO;
    }

    /**
     * 参数对接更新
     *
     * @param param : 对接参数
     * @return : 返回更新结果
     */
    @Override
    public Boolean gameDockUpdate(GameDockParam param) {
        if (Strings.isNotBlank(param.getAppKey())) {
            super.update(new LambdaUpdateWrapper<Game>()
                    .set(Game::getAppKey, param.getAppKey())
                    .set(Game::getUpdateTime, LocalDateTime.now())
                    .eq(Game::getId, param.getId()));
        }
        if (Strings.isBlank(param.getAppId()) && Strings.isBlank(param.getAppSecret())) {
            return Boolean.TRUE;
        }
        GameApplet gameApplet = gameAppletService.getOne(new LambdaQueryWrapper<GameApplet>()
                .eq(GameApplet::getGameId, param.getId()));
        if (gameApplet == null) {
            gameApplet = GameApplet.builder()
                    .gameId(param.getId())
                    .createTime(LocalDateTime.now())
                    .build();
        }
        gameApplet.setUpdateTime(LocalDateTime.now());
        if (Strings.isNotBlank(param.getAppId())) {
            gameApplet.setAppId(param.getAppId());
        }
        if (Strings.isNotBlank(param.getAppSecret())) {
            gameApplet.setAppSecret(param.getAppSecret());
        }
        return gameAppletService.saveOrUpdate(gameApplet);
    }

    /**
     * 获取游戏关联信息
     *
     * @param id : 游戏id
     * @return : 返回游戏关联信息
     */
    @Override
    public GameRelationVO getGameRelation(Long id) {
        Game game = super.getById(id);
        GameRelationVO gameRelationVO = BeanUtils.copy(game, GameRelationVO.class);
        if (gameRelationVO == null) {
            return null;
        }
        Map<Long, Game> gameMap = this.getGameRelationMap(Collections.singletonList(game));
        //没有关联游戏
        if (gameMap.isEmpty()) {
            return gameRelationVO;
        }
        //设置关联游戏名字
        if (Objects.equals(gameRelationVO.getParentId(), 0L)) {
            gameRelationVO.setParentId(game.getId());
            gameRelationVO.setParentName(game.getName());
            gameRelationVO.setParentCategoryId(game.getCategory());
        } else {
            //非主游戏
            if (gameRelationVO.getParentId() != null && gameMap.containsKey(gameRelationVO.getParentId())) {
                gameRelationVO.setParentName(gameMap.get(gameRelationVO.getParentId()).getName());
                gameRelationVO.setParentCategoryId(gameMap.get(gameRelationVO.getParentId()).getCategory());
            }
        }
        if (gameRelationVO.getGuideGameId() != null && gameMap.containsKey(gameRelationVO.getGuideGameId())) {
            gameRelationVO.setGuideGameName(gameMap.get(gameRelationVO.getGuideGameId()).getName());
            gameRelationVO.setGuideCategoryId(gameMap.get(gameRelationVO.getGuideGameId()).getCategory());
        }
        if (gameRelationVO.getH5GameId() != null && gameMap.containsKey(gameRelationVO.getH5GameId())) {
            gameRelationVO.setH5GameName(gameMap.get(gameRelationVO.getH5GameId()).getName());
        }
        //设置关联游戏应用类型名字
        Set<Long> categoryIdSet = new HashSet<>();
        if (gameRelationVO.getParentCategoryId() != null) {
            categoryIdSet.add(gameRelationVO.getParentCategoryId());
        }
        if (gameRelationVO.getGuideCategoryId() != null) {
            categoryIdSet.add(gameRelationVO.getGuideCategoryId());
        }
        Map<Long, GameCategory> categoryMap = gameCategoryService.listByIds(categoryIdSet).stream()
                .collect(Collectors.toMap(GameCategory::getId, Function.identity()));
        if (gameRelationVO.getParentCategoryId() != null && categoryMap.containsKey(gameRelationVO.getParentCategoryId())) {
            gameRelationVO.setParentCategoryName(categoryMap.get(gameRelationVO.getParentCategoryId()).getName());
        }
        if (gameRelationVO.getGuideCategoryId() != null && categoryMap.containsKey(gameRelationVO.getGuideCategoryId())) {
            gameRelationVO.setGuideCategoryName(categoryMap.get(gameRelationVO.getGuideCategoryId()).getName());
        }
        return gameRelationVO;
    }

    /**
     * 获取游戏的关联map
     *
     * @param gameList : 游戏列表
     * @return : 返回游戏列表
     */
    private Map<Long, Game> getGameRelationMap(List<Game> gameList) {
        Set<Long> gameIdSet = new HashSet<>();
        gameList.forEach(game -> {
            if (game.getParentId() != null && !Objects.equals(game.getParentId(), 0L)) {
                gameIdSet.add(game.getParentId());
            }
            if (game.getH5GameId() != null) {
                gameIdSet.add(game.getH5GameId());
            }
            if (game.getGuideGameId() != null) {
                gameIdSet.add(game.getGuideGameId());
            }
        });
        if (gameIdSet.isEmpty()) {
            return Collections.emptyMap();
        }
        return super.listByIds(gameIdSet).stream().collect(Collectors.toMap(Game::getId, Function.identity()));
    }

    /**
     * 导量游戏更新
     *
     * @param param : 关联游戏更新参数
     * @return : 返回更新结果
     */
    @Override
    public Boolean guideGameUpdateUpdate(GuideGameUpdateParam param) {
        return super.update(new LambdaUpdateWrapper<Game>()
                .set(param.getGuideGameId() != null, Game::getGuideGameId, param.getGuideGameId())
                .set(Game::getUpdateTime, LocalDateTime.now())
                .eq(Game::getId, param.getId()));
    }

    /**
     * 关联游戏更新
     *
     * @param param : 关联游戏更新参数
     * @return : 返回更新结果
     */
    @Override
    public Boolean relationGameUpdate(RelationGameUpdateParam param) {
        if (param.getParentId() == null && param.getH5GameId() == null) {
            return Boolean.FALSE;
        }
        return super.update(new LambdaUpdateWrapper<Game>()
                .set(param.getParentId() != null, Game::getParentId, param.getParentId())
                .set(param.getH5GameId() != null, Game::getH5GameId, param.getH5GameId())
                .set(Game::getUpdateTime, LocalDateTime.now())
                .eq(Game::getId, param.getId()));
    }

    /**
     * 获取游戏基本信息
     *
     * @param gameId : 游戏id
     * @return : 返回游戏信息
     */
    @Override
    public GameInfoVO getGameInfo(Long gameId) {
        Game game = super.getById(gameId);
        GameInfoVO gameInfoVO = BeanUtils.copy(game, GameInfoVO.class);
        if (gameInfoVO == null) {
            return null;
        }
        if (gameInfoVO.getCpId() != null) {
            Cp cp = cpService.getById(gameInfoVO.getCpId());
            if (cp != null) {
                gameInfoVO.setCpName(cp.getCpName());
            }
        }
        if (Strings.isNotBlank(game.getClassify())) {
            String[] split = game.getClassify().split(",");
            List<Long> idList = Arrays.stream(split).map(Long::valueOf).collect(Collectors.toList());
            List<GameCategoryVO> gameCategoryList = gameCategoryService.listByIdList(idList, GameCategoryEnum.GAME_LABEL.getCategoryType());
            gameInfoVO.setClassifyList(BeanUtils.copyList(gameCategoryList, GameCategoryParentVO.class));
        }
        if (game.getCategory() != null) {
            GameCategoryVO gameCategoryVO = gameCategoryService.getById(game.getCategory());
            gameInfoVO.setCategory(BeanUtils.copy(gameCategoryVO, GameCategoryParentVO.class));
        }
        //是否主游戏
        gameInfoVO.setIsParentGame(Objects.equals(gameInfoVO.getParentId(), 0L));
        //关联游戏查询
        Map<Long, Game> gameMap = this.getGameRelationMap(Collections.singletonList(game));
        //没有关联游戏
        if (gameMap.isEmpty()) {
            return gameInfoVO;
        }
        //设置关联游戏名字
        if (gameInfoVO.getParentId() != null && !Objects.equals(gameInfoVO.getParentId(), 0L)) {
            gameInfoVO.setParentName(gameMap.get(gameInfoVO.getParentId()).getName());
        } else {
            gameInfoVO.setParentId(game.getId());
            gameInfoVO.setParentName(game.getName());
        }
        if (gameInfoVO.getH5GameId() != null) {
            gameInfoVO.setH5GameName(gameMap.get(gameInfoVO.getH5GameId()).getName());
        }
        if (gameInfoVO.getGuideGameId() != null) {
            gameInfoVO.setGuideGameName(gameMap.get(gameInfoVO.getGuideGameId()).getName());
        }
        return gameInfoVO;
    }

    /**
     * 游戏基本信息更新
     *
     * @param param : 游戏更新参数
     * @return : 返回更新结果
     */
    @Override
    public Boolean updateGameInfo(GameUpdateParam param) {
        //游戏分类处理
        List<Long> classifyList = param.getClassifyList();
        Map<String, String> map = this.getClassifyMap(classifyList);
        return super.update(new LambdaUpdateWrapper<Game>()
                .set(Strings.isNotBlank(param.getName()), Game::getName, param.getName())
                .set(param.getCpId() != null, Game::getCpId, param.getCpId())
                .set(Strings.isNotBlank(map.get("classify")), Game::getClassify, map.get("classify"))
                .set(Strings.isNotBlank(map.get("classifyParent")), Game::getClassifyParent, map.get("classifyParent"))
                .set(param.getCategory() != null, Game::getCategory, param.getCategory())
                .set(param.getShareScale() != null, Game::getShareScale, param.getShareScale())
                .set(Objects.equals(param.getIsParentGame(), Boolean.TRUE), Game::getParentId, 0L)
                .set(!Objects.equals(param.getIsParentGame(), Boolean.TRUE) && param.getParentId() != null,
                        Game::getParentId, param.getParentId())
                .set(param.getH5GameId() != null, Game::getH5GameId, param.getH5GameId())
                .set(param.getGuideGameId() != null, Game::getGuideGameId, param.getGuideGameId())
                .set(Strings.isNotBlank(param.getPublicity()), Game::getPublicity, param.getPublicity())
                .set(Strings.isNotBlank(param.getDescription()), Game::getDescription, param.getDescription())
                .set(Strings.isNotBlank(param.getVersion()), Game::getVersion, param.getVersion())
                .set(Strings.isNotBlank(param.getGameUrl()), Game::getGameUrl, param.getGameUrl())
                .set(Strings.isNotBlank(param.getCpPaybackUrl()), Game::getCpPaybackUrl, param.getCpPaybackUrl())
                .set(Game::getUpdateTime, LocalDateTime.now())
                .eq(Game::getId, param.getId()));
    }

    /**
     * 新增游戏
     *
     * @param param : 游戏新增参数
     * @return : 返回添加结果
     */
    @Override
    public Boolean gameAdd(GameAddParam param) {
        //游戏分类处理
        List<Long> classifyList = param.getClassifyList();
        Map<String, String> map = this.getClassifyMap(classifyList);
        Game game = Game.builder()
                .cpId(param.getCpId())
                .name(param.getName())
                .shareScale(param.getShareScale())
                .category(param.getCategory())
                .classify(map.get("classify"))
                .classifyParent(map.get("classifyParent"))
                .createTime(LocalDateTime.now())
                .updateTime(LocalDateTime.now())
                .build();
        if (!Objects.equals(param.getIsParentGame(), Boolean.TRUE) && param.getParentGameId() != null) {
            game.setParentId(param.getParentGameId());
        }
        return super.save(game);
    }

    /**
     * 分类处理
     *
     * @param classifyList : 选的游戏分类列表
     * @return : 返回分类信息
     */
    private Map<String, String> getClassifyMap(List<Long> classifyList) {
        if (CollectionUtils.isEmpty(classifyList)) {
            return Collections.emptyMap();
        }
        List<GameCategoryVO> gameCategoryVOList = gameCategoryService.listByIdList(classifyList, GameCategoryEnum.GAME_LABEL.getCategoryType());
        Set<Long> classifySet = new HashSet<>();
        Set<Long> classifyParentSet = new HashSet<>();
        gameCategoryVOList.forEach(vo -> {
            classifySet.add(vo.getId());
            if (Objects.equals(vo.getParentId(), 0L)) {
                classifyParentSet.add(vo.getId());
            }
        });
        Map<String, String> map = new HashMap<>(2);
        if (!classifySet.isEmpty()) {
            map.put("classify", StringUtils.join(classifySet, ","));
        }
        if (!classifyParentSet.isEmpty()) {
            map.put("classifyParent", StringUtils.join(classifySet, ","));
        }
        return map;
    }

    /**
     * 查询游戏列表
     *
     * @param param : 游戏列表查询参数
     * @return : 返回单页游戏游戏列表数据
     */
    @Override
    public IPage<GameListVO> gameList(GameListParam param) {
        //cp列表
        Map<Long, String> cpMap = cpService.choiceList().stream()
                .collect(Collectors.toMap(CpChoiceVO::getId, CpChoiceVO::getCpName));
        //游戏分类列表
        Map<Long, GameCategoryDTO> gameCategoryMap = gameCategoryService.gameCategoryMap();
        //查询游戏
        Map<Long, String> gameMap = this.choiceList().stream()
                .collect(Collectors.toMap(GameChoiceVO::getId, GameChoiceVO::getName));
        //执行查询
        return page(param.toPage(), new QueryWrapper<Game>().lambda()
                .eq(param.getCpId() != null, Game::getCpId, param.getCpId())
                .eq(param.getGameId() != null && !Objects.equals(param.getIsParentGame(), Boolean.TRUE), Game::getId, param.getGameId())
                .and(param.getGameId() != null && Objects.equals(param.getIsParentGame(), Boolean.TRUE),
                        qw -> qw.eq(Game::getId, param.getGameId()).or().eq(Game::getParentId, param.getGameId()))
                .eq(param.getGameCategoryId() != null, Game::getCategory, param.getGameCategoryId())
                .eq(param.getH5GameId() != null, Game::getH5GameId, param.getH5GameId())
                .eq(param.getGuideGameId() != null, Game::getGuideGameId, param.getGuideGameId())
                .apply(param.getGameClassifyId() != null && !Objects.equals(param.getIsParentClassify(), Boolean.TRUE),
                        "FIND_IN_SET({0}, classify)", String.valueOf(param.getGameClassifyId()))
                .apply(param.getGameClassifyId() != null && Objects.equals(param.getIsParentClassify(), Boolean.TRUE),
                        "FIND_IN_SET({0}, classifyParent)", String.valueOf(param.getGameClassifyId()))
                .eq(param.getStatus() != null, Game::getStatus, param.getStatus())
                .orderByDesc(Game::getCreateTime)
        ).convert(game -> this.toVo(game, cpMap, gameCategoryMap, gameMap));
    }

    /**
     * 游戏对象信息转换
     *
     * @param game : 游戏信息
     * @return : 返回展示对象
     */
    private GameListVO toVo(Game game, Map<Long, String> cpMap, Map<Long, GameCategoryDTO> gameCategoryMap, Map<Long, String> gameMap) {
        GameListVO gameListVO = BeanUtils.copy(game, GameListVO.class);
        if (gameListVO == null) {
            return null;
        }
        //cp设置
        if (gameListVO.getCpId() != null) {
            gameListVO.setCpName(cpMap.get(gameListVO.getCpId()));
        }
        //游戏类型
        if (gameListVO.getCategory() != null) {
            gameListVO.setCategoryName(gameCategoryMap.get(gameListVO.getCategory()) == null ? null : gameCategoryMap.get(gameListVO.getCategory()).getName());
        }
        //游戏分类
        if (gameListVO.getClassify() != null) {
            String[] classifyIds = gameListVO.getClassify().split(",");
            List<String> classifyNameList = new ArrayList<>();
            for (String classifyId : classifyIds) {
                classifyNameList.add(gameCategoryMap.get(Long.valueOf(classifyId)) == null ? null : gameCategoryMap.get(Long.valueOf(classifyId)).getName());
            }
            gameListVO.setClassifyNameList(classifyNameList);
        }
        //主游戏设置
        if (Objects.equals(game.getParentId(), 0L)) {
            gameListVO.setIsParentGame(Boolean.TRUE);
            gameListVO.setParentId(game.getId());
            gameListVO.setParentName(game.getName());
        } else {
            gameListVO.setIsParentGame(Boolean.FALSE);
            gameListVO.setParentName(gameMap.get(gameListVO.getParentId()));
        }
        //h5游戏
        if (gameListVO.getH5GameId() != null) {
            gameListVO.setH5GameName(gameMap.get(gameListVO.getH5GameId()));
        }
        //关联游戏
        if (gameListVO.getGuideGameId() != null) {
            gameListVO.setGuideGameName(gameMap.get(gameListVO.getGuideGameId()));
        }
        //查询支付方式列表
        List<GamePayWayVO> gamePayWayVOList = gamePayWayService.getByGameId(game.getId());
        gameListVO.setGamePayWayList(gamePayWayVOList);
        //查询规则配置列表
        List<GameStrategyVO> gameStrategyVOList = gameStrategyService.getByGameId(game.getId());
        gameListVO.setGameStrategyList(gameStrategyVOList);
        return gameListVO;
    }

    /**
     * 游戏选择列表查询
     *
     * @return : 返回所有游戏信息
     */
    @Override
    public List<GameChoiceVO> choiceList() {
        List<Game> gameList = super.list(new LambdaQueryWrapper<Game>().select(Game::getId, Game::getName, Game::getCategory));
        return BeanUtils.copyList(gameList, GameChoiceVO.class);
    }

    /**
     * 根据id查询游戏信息
     *
     * @param id : 游戏id
     * @return : 返回游戏信息
     */
    @Override
    public GameDTO getById(Long id) {
        if (id == null) {
            return null;
        }
        return BeanUtils.copy(super.getById(id), GameDTO.class);
    }

    /**
     * 游戏相关条件查询游戏信息
     *
     * @param cpId           : cp唯一键
     * @param gameId         : 游戏id
     * @param gameCategoryId : 分类id
     * @return : 返回游戏信息列表
     */
    @Override
    public Map<Long, GameDTO> gameCondition(Long cpId, Long gameId, Long gameCategoryId) {
        //游戏条件判断
        if (cpId == null && gameId == null && gameCategoryId == null) {
            return null;
        }
        List<GameDTO> gameList = this.getGameList(cpId, gameId, gameCategoryId);
        if (CollectionUtils.isEmpty(gameList)) {
            return Collections.emptyMap();
        }
        return gameList.stream().collect(Collectors.toMap(GameDTO::getId, Function.identity()));
    }

    /**
     * 游戏查询条件
     *
     * @param cpId           : cp唯一键
     * @param gameId         : 游戏id
     * @param gameCategoryId : 分类id
     * @return : 返回游戏信息列表, null 标识未匹配到游戏, 空表示没有条件
     */
    private List<GameDTO> getGameList(Long cpId, Long gameId, Long gameCategoryId) {
        if (cpId == null && gameId == null && gameCategoryId == null) {
            return Collections.emptyList();
        }
        List<Game> gameList = super.list(new LambdaQueryWrapper<Game>()
                .eq(cpId != null, Game::getCpId, cpId)
                .eq(gameId != null, Game::getId, gameId)
                .eq(gameCategoryId != null, Game::getCategory, gameCategoryId)
                .select(Game::getId, Game::getCpId, Game::getName, Game::getCategory)
        );
        return BeanUtils.copyList(gameList, GameDTO.class);
    }

    /**
     * 根据主键id删除
     *
     * @param id : 主键id
     * @return : 返回删除结果
     */
    @Override
    public Boolean deleteById(Long id) {
        return super.removeById(id);
    }

    /**
     * 根据主键id删除
     *
     * @param id : 主键id
     * @return : 返回删除结果
     */
    @Override
    public Boolean statusUpdate(Long id, Integer status) {
        return super.update(new LambdaUpdateWrapper<Game>()
                .set(Game::getStatus, status)
                .set(Game::getUpdateTime, LocalDateTime.now())
                .eq(Game::getId, id));
    }

}
