package com.zanxiang.manage.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.enums.DeleteEnum;
import com.zanxiang.common.enums.StatusEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.manage.domain.bo.PayApplicationBO;
import com.zanxiang.manage.domain.params.PayApplicationListNoPageParam;
import com.zanxiang.manage.domain.params.PayApplicationListParam;
import com.zanxiang.manage.domain.vo.PayApplicationVO;
import com.zanxiang.manage.service.PayApplicationService;
import com.zanxiang.manage.service.api.MiniAppletApiService;
import com.zanxiang.mybatis.entity.PayApplication;
import com.zanxiang.mybatis.mapper.PayApplicationMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;

/**
 * 游戏应用表 服务实现类
 *
 * @author xufeng
 * @date 2022-06-20 15:16
 */
@Service
public class PayApplicationServiceImpl extends ServiceImpl<PayApplicationMapper, PayApplication> implements PayApplicationService {

    @Autowired
    private MiniAppletApiService miniAppletApiService;

    @Override
    public Boolean saveOrUpdate(PayApplicationBO bo) {
        PayApplication app = BeanUtils.copy(bo, PayApplication.class);
        if (bo.getIsDelete() != null && bo.getIsDelete() == DeleteEnum.YES.getCode()) {
            return removeById(bo.getId());
        }
        return saveOrUpdate(app);
    }

    @Override
    public IPage<PayApplicationVO> list(PayApplicationListParam bo) {
        PayApplication app = BeanUtils.copy(bo, PayApplication.class);
        return page(bo.toPage(), new LambdaQueryWrapper<PayApplication>().setEntity(app).orderByDesc(PayApplication::getCreateTime)).convert(this::toVo);
    }

    @Override
    public List<PayApplicationVO> listNoPage(PayApplicationListNoPageParam bo) {
        PayApplication app = BeanUtils.copy(bo, PayApplication.class);
        app.setStatus(StatusEnum.YES.getCode());
        List<PayApplication> list = list(new LambdaQueryWrapper<PayApplication>().setEntity(app).orderByDesc(PayApplication::getCreateTime));
        if (list.size() == 0) {
            return null;
        }
        return BeanUtils.copyList(list, PayApplicationVO.class);
    }

    private PayApplicationVO toVo(PayApplication payApplication) {
        if (Objects.isNull(payApplication)) {
            return null;
        }
        return BeanUtils.copy(payApplication, PayApplicationVO.class);
    }

    /**
     * 获取游戏应用二维码图片地址
     *
     * @param id : 应用id
     * @return : 返回游戏二维码
     */
    @Override
    public String getQrCode(Long id) {
        PayApplication payApplication = super.getById(id);
        if (payApplication == null) {
            throw new BaseException("应用信息不存在");
        }
        return miniAppletApiService.getQrCode(payApplication.getAppId(), payApplication.getAppSecret(), payApplication.getName());
    }

}
