package com.zanxiang.manage.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.manage.domain.dto.PromoAccountDTO;
import com.zanxiang.manage.domain.vo.PromoAccountChoiceVO;
import com.zanxiang.manage.domain.vo.PromoPitcherChoiceVO;
import com.zanxiang.manage.service.PromoAccountService;
import com.zanxiang.mybatis.entity.PromoAccount;
import com.zanxiang.mybatis.mapper.PromoAccountMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2022-11-03
 * @description : 推广账号
 */
@Slf4j
@Service
public class PromoAccountServiceImpl extends ServiceImpl<PromoAccountMapper, PromoAccount> implements PromoAccountService {

    /**
     * 推广账号选择列表查询
     *
     * @return : 返回所有推广账号信息
     */
    @Override
    public List<PromoAccountChoiceVO> promoAccountChoiceList() {
        List<PromoAccount> promoAccountList = super.list(new LambdaQueryWrapper<PromoAccount>().
                select(PromoAccount::getAccount, PromoAccount::getName));
        return BeanUtils.copyList(promoAccountList, PromoAccountChoiceVO.class);
    }

    /**
     * 投手选择列表查询
     *
     * @return : 返回所有投手信息
     */
    @Override
    public List<PromoPitcherChoiceVO> promoPitcherChoiceList() {
        List<PromoAccount> promoAccountList = super.list(new LambdaQueryWrapper<PromoAccount>().
                select(PromoAccount::getPitcherId, PromoAccount::getPitcherName));
        return BeanUtils.copyList(promoAccountList, PromoPitcherChoiceVO.class);
    }

    /**
     * 注册渠道信息
     *
     * @return : 返回所有渠道信息
     */
    @Override
    public Map<Long, PromoAccountDTO> promoAccountMap() {
        List<PromoAccount> promoChannelList = super.list();
        if (CollectionUtils.isEmpty(promoChannelList)) {
            return Collections.emptyMap();
        }
        List<PromoAccountDTO> promoAccountDTOList = BeanUtils.copyList(promoChannelList, PromoAccountDTO.class);
        return promoAccountDTOList.stream().collect(Collectors.toMap(PromoAccountDTO::getId, Function.identity()));
    }

    /**
     * 通过帐户或投手id列表
     *
     * @param account   账户
     * @param pitcherId 投手id
     * @return {@link List}<{@link Long}>
     */
    @Override
    public List<Long> listByAccountOrPitcherId(String account, String pitcherId) {
        List<PromoAccount> promoAccountList = super.list(new LambdaQueryWrapper<PromoAccount>()
                .eq(Strings.isNotBlank(account), PromoAccount::getAccount, account)
                .eq(Strings.isNotBlank(pitcherId), PromoAccount::getPitcherId, pitcherId));
        if (CollectionUtils.isEmpty(promoAccountList)) {
            return Collections.emptyList();
        }
        return promoAccountList.stream().map(PromoAccount::getId).collect(Collectors.toList());
    }
}
