package com.zanxiang.manage.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.manage.domain.dto.PromoChannelDTO;
import com.zanxiang.manage.domain.vo.PromoChannelChoiceVO;
import com.zanxiang.manage.service.PromoAccountService;
import com.zanxiang.manage.service.PromoChannelService;
import com.zanxiang.mybatis.entity.PromoChannel;
import com.zanxiang.mybatis.mapper.PromoChannelMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2022-10-31
 * @description : 推广渠道
 */
@Slf4j
@Service
public class PromoChannelServiceImpl extends ServiceImpl<PromoChannelMapper, PromoChannel> implements PromoChannelService {

    @Autowired
    private PromoAccountService promoAccountService;

    /**
     * 注册渠道选择列表查询
     *
     * @return : 返回所有注册渠道信息
     */
    @Override
    public List<PromoChannelChoiceVO> promoChannelChoiceList() {
        List<PromoChannel> promoChannelList = super.list(new LambdaQueryWrapper<PromoChannel>().
                select(PromoChannel::getId, PromoChannel::getChannel));
        return BeanUtils.copyList(promoChannelList, PromoChannelChoiceVO.class);
    }

    /**
     * 注册渠道信息
     *
     * @return : 返回所有渠道信息
     */
    @Override
    public Map<Long, PromoChannelDTO> promoChannelMap() {
        List<PromoChannel> promoChannelList = super.list(new LambdaQueryWrapper<PromoChannel>().
                select(PromoChannel::getId, PromoChannel::getChannel, PromoChannel::getAccountId));
        if (CollectionUtils.isEmpty(promoChannelList)) {
            return Collections.emptyMap();
        }
        List<PromoChannelDTO> promoChannelDTOList = BeanUtils.copyList(promoChannelList, PromoChannelDTO.class);
        return promoChannelDTOList.stream().collect(Collectors.toMap(PromoChannelDTO::getId, Function.identity()));
    }

    /**
     * 通过帐户或投手id列表
     *
     * @param account   账户
     * @param pitcherId 投手id
     * @return {@link List}<{@link Long}>
     */
    @Override
    public List<Long> listByAccountOrPitcherId(String account, String pitcherId) {
        if (Strings.isBlank(account) && Strings.isBlank(pitcherId)) {
            return null;
        }
        List<Long> accountIdList = promoAccountService.listByAccountOrPitcherId(account, pitcherId);
        if (CollectionUtils.isEmpty(accountIdList)) {
            return Collections.emptyList();
        }
        List<PromoChannel> promoChannelList = super.list(new LambdaQueryWrapper<PromoChannel>()
                .in(PromoChannel::getAccountId, accountIdList));
        if (CollectionUtils.isEmpty(promoChannelList)) {
            return Collections.emptyList();
        }
        return promoChannelList.stream().map(PromoChannel::getId).collect(Collectors.toList());
    }
}
