package com.zanxiang.manage.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.manage.domain.params.UserNameAuthListParam;
import com.zanxiang.manage.domain.vo.UserCardVO;
import com.zanxiang.manage.service.UserCardService;
import com.zanxiang.module.util.bean.BeanUtil;
import com.zanxiang.mybatis.entity.UserCard;
import com.zanxiang.mybatis.mapper.UserCardMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.Objects;

/**
 * @author : lingfeng
 * @time : 2022-09-26
 * @description : 用户实名认证
 */
@Slf4j
@Service
public class UserCardServiceImpl extends ServiceImpl<UserCardMapper, UserCard> implements UserCardService {

    /**
     * 获取玩家实名认证列表
     *
     * @param param : 查询参数
     * @return : 返回单页数据
     */
    @Override
    public IPage<UserCardVO> getUserCardList(UserNameAuthListParam param) {
        return page(param.toPage(), new QueryWrapper<UserCard>().lambda()
                .eq(param.getUserId() != null, UserCard::getUserId, param.getUserId())
                .eq(Strings.isNotBlank(param.getUserName()), UserCard::getUsername, param.getUserName())
                .eq(Strings.isNotBlank(param.getNickname()), UserCard::getNickname, param.getNickname())
                .ge(param.getBeginDate() != null, UserCard::getRegTime, param.getBeginDate() == null ? null : LocalDateTime.of(param.getBeginDate(), LocalTime.MIN))
                .le(param.getEndDate() != null, UserCard::getRegTime, param.getEndDate() == null ? null : LocalDateTime.of(param.getEndDate(), LocalTime.MAX))
                .eq(Strings.isNotBlank(param.getRealName()), UserCard::getCardName, param.getRealName())
                .orderByDesc(UserCard::getCreateTime))
                .convert(this::toVo);
    }

    /**
     * 对象转换
     *
     * @param userCard 用户卡片
     * @return {@link UserCardVO}
     */
    private UserCardVO toVo(UserCard userCard) {
        if (Objects.isNull(userCard)) {
            return null;
        }
        UserCardVO userCardVO = BeanUtil.copy(userCard, UserCardVO.class);
        userCardVO.setCardId(userCard.getShowCardId());
        userCardVO.setCardName(userCard.getShowCardName());
        return userCardVO;
    }

    /**
     * 通过用户id
     *
     * @param userId 用户id
     * @return {@link UserCardVO}
     */
    @Override
    public UserCardVO getByUserId(Long userId) {
        UserCard userCard = this.getOne(new LambdaQueryWrapper<UserCard>()
                .eq(UserCard::getUserId, userId));
        return this.toVo(userCard);
    }

}
