package com.zanxiang.manage.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.manage.domain.dto.GameDTO;
import com.zanxiang.manage.domain.dto.GameUserRoleDTO;
import com.zanxiang.manage.domain.params.UserLoginLogParam;
import com.zanxiang.manage.domain.vo.GameCategoryVO;
import com.zanxiang.manage.domain.vo.UserRoleLoginVO;
import com.zanxiang.manage.service.GameCategoryService;
import com.zanxiang.manage.service.GameService;
import com.zanxiang.manage.service.GameUserRoleService;
import com.zanxiang.manage.service.UserLoginLogService;
import com.zanxiang.mybatis.entity.UserLoginLog;
import com.zanxiang.mybatis.mapper.UserLoginLogMapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @author : lingfeng
 * @time : 2022-07-05
 * @description : 用户登录日志
 */
@Slf4j
@Service
public class UserLoginLogServiceImpl extends ServiceImpl<UserLoginLogMapper, UserLoginLog> implements UserLoginLogService {

    @Autowired
    private GameUserRoleService gameUserRoleService;

    @Autowired
    private GameService gameService;

    @Autowired
    private GameCategoryService gameCategoryService;

    /**
     * 查询用户登录记录
     *
     * @param param : 登录记录查询参数
     * @return : 返回单页数据
     */
    @Override
    public IPage<UserRoleLoginVO> loginList(UserLoginLogParam param) {
        //执行查询
        return page(param.toPage(), new QueryWrapper<UserLoginLog>().lambda()
                .eq(UserLoginLog::getUserId, param.getUserId())
                .orderByDesc(UserLoginLog::getCreateTime)
        ).convert(this::toVo);
    }

    /**
     * 玩家角色登录信息对象转换
     *
     * @param userLoginLog : 玩家角色登录信息
     * @return : 玩家角色登录信息展示对象
     */
    private UserRoleLoginVO toVo(UserLoginLog userLoginLog) {
        UserRoleLoginVO roleLoginVO = BeanUtils.copy(userLoginLog, UserRoleLoginVO.class);
        if (userLoginLog == null) {
            return null;
        }
        roleLoginVO.setLoginIp(userLoginLog.getIp());
        roleLoginVO.setLoginTime(userLoginLog.getCreateTime());
        //查询角色名称
        GameUserRoleDTO gameUserRoleDTO = gameUserRoleService.getByRoleId(userLoginLog.getRoleId());
        if (gameUserRoleDTO != null) {
            roleLoginVO.setRoleName(gameUserRoleDTO.getRoleName());
        }
        //设置游戏名称
        GameDTO gameDTO = gameService.getById(userLoginLog.getGameId());
        if (gameDTO != null) {
            //设置游戏列表
            GameCategoryVO gameCategoryVO = gameCategoryService.getById(gameDTO.getCategory());
            if (gameCategoryVO != null) {
                roleLoginVO.setGameCategoryName(gameCategoryVO.getName());
            }
            roleLoginVO.setGameName(gameDTO.getName());
        }
        return roleLoginVO;
    }
}
