package com.zanxiang.manage.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.constant.Constants;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.utils.MD5Util;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.manage.domain.dto.*;
import com.zanxiang.manage.domain.params.UserListParam;
import com.zanxiang.manage.domain.params.UserUpdateParam;
import com.zanxiang.manage.domain.vo.GameUserVO;
import com.zanxiang.manage.domain.vo.UserCardVO;
import com.zanxiang.manage.domain.vo.UserListVO;
import com.zanxiang.manage.domain.vo.UserVO;
import com.zanxiang.manage.service.*;
import com.zanxiang.mybatis.entity.PromoAccount;
import com.zanxiang.mybatis.entity.PromoChannel;
import com.zanxiang.mybatis.entity.User;
import com.zanxiang.mybatis.entity.UserExt;
import com.zanxiang.mybatis.mapper.UserMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.logging.log4j.util.Strings;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * @author : lingfeng
 * @time : 2022-06-07
 * @description : 用户玩家
 */
@Slf4j
@Service
public class UserServiceImpl extends ServiceImpl<UserMapper, User> implements UserService {

    @Autowired
    private UserExtService userExtService;

    @Autowired
    private GameService gameService;

    @Autowired
    private GameUserService gameUserService;

    @Autowired
    private PromoChannelService promoChannelService;

    @Autowired
    private PromoAccountService promoAccountService;

    @Autowired
    private UserService userService;

    @Autowired
    private CpService cpService;

    @Autowired
    private GameCategoryService gameCategoryService;

    @Autowired
    private UserCardService userCardService;

    /**
     * 玩家列表查询
     *
     * @param param : 查询参数
     * @return : 返回单页玩家数据
     */
    @Override
    public IPage<UserListVO> list(UserListParam param) {
        //渠道列表
        List<Long> agentIdList;
        if (param.getChannelId() == null) {
            agentIdList = promoChannelService.listByAccountOrPitcherId(param.getAccountId(), param.getPitcherId());
        } else {
            agentIdList = Collections.singletonList(param.getChannelId());
        }
        //根据条件, 匹配渠道
        if (agentIdList != null && agentIdList.isEmpty()) {
            return new Page<>();
        }
        //游戏条件处理
        Map<Long, GameDTO> gameMap = gameService.gameCondition(param.getCpId(), param.getGameId(), param.getGameCategoryId());
        //根据条件, 匹配不到游戏
        if (gameMap != null && gameMap.isEmpty()) {
            return new Page<>();
        }
        //玩家条件处理
        Map<Long, UserDTO> userMap = userService.userCondition(param.getUserId(), agentIdList, param.getUserName(),
                param.getNickname(), param.getRegIp(), param.getAdId());
        //根据条件, 匹配不到玩家
        if (userMap != null && userMap.isEmpty()) {
            return new Page<>();
        }
        //相关用户id
        Set<Long> userIdSet = userMap == null ? null : userMap.keySet();
        Map<Long, CpDTO> cpMap = cpService.cpMap();
        Map<Long, PromoChannelDTO> channelMap = promoChannelService.promoChannelMap();
        Map<Long, GameCategoryDTO> gameCategoryMap = gameCategoryService.gameCategoryMap();
        Map<Long, PromoAccountDTO> promoAccountMap = promoAccountService.promoAccountMap();
        //执行查询
        return page(param.toPage(), new QueryWrapper<User>().lambda()
                .eq(param.getUserId() != null, User::getId, param.getUserId())
                .like(Strings.isNotBlank(param.getUserName()), User::getUsername, param.getUserName())
                .like(Strings.isNotBlank(param.getNickname()), User::getNickname, param.getNickname())
                .eq(Strings.isNotBlank(param.getMobile()), User::getMobile, param.getMobile())
                .in(CollectionUtils.isNotEmpty(userIdSet), User::getId, userIdSet)
                .isNotNull(Objects.equals(param.getIsBindMobile(), Boolean.TRUE), User::getMobile)
                .isNull(Objects.equals(param.getIsBindMobile(), Boolean.FALSE), User::getMobile)
                .eq(Objects.equals(param.getIsAuth(), Boolean.FALSE), User::getAuthentication, 0)
                .ne(Objects.equals(param.getIsAuth(), Boolean.TRUE), User::getAuthentication, 0)
                .in(CollectionUtils.isNotEmpty(gameMap), User::getGameId, CollectionUtils.isEmpty(gameMap) ? null : gameMap.keySet())
                .ge(param.getBeginDate() != null, User::getCreateTime, param.getBeginDate() == null ? null : LocalDateTime.of(param.getBeginDate(), LocalTime.MIN))
                .le(param.getEndDate() != null, User::getCreateTime, param.getEndDate() == null ? null : LocalDateTime.of(param.getEndDate(), LocalTime.MAX))
                .ge(param.getRechargeBeginDate() != null, User::getLastRechargeTime, param.getRechargeBeginDate() == null ? null : LocalDateTime.of(param.getRechargeBeginDate(), LocalTime.MIN))
                .le(param.getRechargeEndDate() != null, User::getLastRechargeTime, param.getRechargeEndDate() == null ? null : LocalDateTime.of(param.getRechargeEndDate(), LocalTime.MAX))
                .gt(Objects.equals(param.getIsRecharge(), Boolean.TRUE), User::getRechargeCount, 0)
                .eq(Objects.equals(param.getIsRecharge(), Boolean.FALSE), User::getRechargeCount, 0)
                .eq(param.getVipLevel() != null && param.getVipLevel() <= 10, User::getVipMax, param.getVipLevel())
                .gt(param.getVipLevel() != null && param.getVipLevel() > 10, User::getVipMax, param.getVipLevel())
                .eq(param.getStatus() != null, User::getStatus, param.getStatus())
                .eq(param.getChannelId() != null, User::getAgentId, param.getChannelId())
                .eq(param.getCustomerId() != null, User::getCustomerId, param.getCustomerId())
                .eq(param.getIsGs() != null, User::getIsGs, param.getIsGs())
                .orderByDesc(User::getCreateTime)
        ).convert(u -> this.toVo(u, userMap, gameMap, promoAccountMap, channelMap, cpMap, gameCategoryMap));
    }


    /**
     * 给签证官
     *
     * @param user            用户
     * @param userMap         用户映射
     * @param gameMap         游戏地图
     * @param promoAccountMap 推广账户映射
     * @param channelMap      通道映射
     * @param cpMap           cp地图
     * @param gameCategoryMap 游戏类别映射
     * @return {@link UserListVO}
     */
    private UserListVO toVo(User user, Map<Long, UserDTO> userMap, Map<Long, GameDTO> gameMap, Map<Long, PromoAccountDTO> promoAccountMap,
                            Map<Long, PromoChannelDTO> channelMap, Map<Long, CpDTO> cpMap, Map<Long, GameCategoryDTO> gameCategoryMap) {
        UserListVO userListVO = BeanUtils.copy(user, UserListVO.class);
        if (Objects.isNull(userListVO)) {
            return null;
        }
        userListVO.setMobile(user.getShowPhoneNum());
        UserDTO userDTO = StringUtils.isEmpty(userMap) ? null : userMap.get(userListVO.getId());
        if (userDTO == null) {
            userDTO = this.getById(userListVO.getId());
        }
        if (userDTO != null) {
            userListVO.setRegIp(userDTO.getIp());
            userListVO.setUsername(userDTO.getUsername());
            userListVO.setNickname(userDTO.getNickname());
            userListVO.setAgentId(userDTO.getAgentId());
            userListVO.setChannel(channelMap.get(userDTO.getAgentId()) == null ? null : channelMap.get(userDTO.getAgentId()).getChannel());
            userListVO.setAdId(userDTO.getAdId());
        }
        GameDTO gameDTO = StringUtils.isEmpty(gameMap) ? null : gameMap.get(userListVO.getGameId());
        if (gameDTO == null) {
            gameDTO = gameService.getById(userListVO.getGameId());
        }
        if (gameDTO != null) {
            userListVO.setGameName(gameDTO.getName());
            userListVO.setGameCategoryId(gameDTO.getCategory());
            userListVO.setGameCategoryName(gameCategoryMap.get(gameDTO.getCategory()) == null ? null : gameCategoryMap.get(gameDTO.getCategory()).getName());
            userListVO.setCpId(gameDTO.getCpId());
            userListVO.setCpName(cpMap.get(gameDTO.getCpId()) == null ? null : cpMap.get(gameDTO.getCpId()).getCpName());
        }
        PromoChannelDTO promoChannelDTO = channelMap.get(userListVO.getAgentId());
        if (promoChannelDTO != null) {
            PromoAccountDTO promoAccountDTO = promoAccountMap.get(promoChannelDTO.getAccountId());
            if (promoAccountDTO != null) {
                userListVO.setAccountId(promoAccountDTO.getId());
                userListVO.setAccountName(promoAccountDTO.getName());
                userListVO.setPitcherId(promoAccountDTO.getPitcherId());
                userListVO.setPitcherName(promoAccountDTO.getPitcherName());
                userListVO.setMediaId(promoAccountDTO.getMediaId());
                userListVO.setMediaName(promoAccountDTO.getMediaName());
            }
        }
        return userListVO;
    }

    /**
     * 用户信息更新
     *
     * @param param : 用户更新参数
     * @return : 返回更新结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean userUpdate(UserUpdateParam param) {
        //用户信息更新
        if (Strings.isNotBlank(param.getMobile()) || Strings.isNotBlank(param.getAliPay()) || Strings.isNotBlank(param.getPassword())) {
            super.update(new LambdaUpdateWrapper<User>()
                    .eq(User::getId, param.getUserId())
                    .eq(User::getUsername, param.getUsername())
                    .set(Strings.isNotBlank(param.getMobile()), User::getMobile, param.getMobile())
                    .set(Strings.isNotBlank(param.getAliPay()), User::getAliPay, param.getAliPay())
                    .set(Strings.isNotBlank(param.getPassword()), User::getPassword, this.cmfPassword(param.getPassword()))
                    .set(User::getUpdateTime, LocalDateTime.now()));
        }
        //用户拓展信息更新
        if (Strings.isNotBlank(param.getRealName()) || Strings.isNotBlank(param.getIdCard())) {
            userExtService.update(new LambdaUpdateWrapper<UserExt>()
                    .eq(UserExt::getUserId, param.getUserId())
                    .set(Strings.isNotBlank(param.getRealName()), UserExt::getRealName, param.getRealName())
                    .set(Strings.isNotBlank(param.getIdCard()), UserExt::getIdCard, param.getIdCard())
                    .set(UserExt::getUpdateTime, LocalDateTime.now()));
        }
        return Boolean.TRUE;
    }

    /**
     * 密码cfm加密
     *
     * @param password : 密码
     * @return : 返回加密完的密码
     */
    private String cmfPassword(String password) {
        String passwordSalt = password + "ZX_PASSWORD_SALT";
        String charsetName = Constants.UTF8;
        return "###" + MD5Util.MD5Encode(MD5Util.MD5Encode(passwordSalt, charsetName), charsetName);
    }

    /**
     * 查询玩家详细信息
     *
     * @param userId : 用户id
     * @return : 返回用户信息
     */
    @Override
    public UserVO getUserInfo(Long userId) {
        //查询用户信息
        User user = super.getById(userId);
        UserVO userVO = BeanUtils.copy(user, UserVO.class);
        if (userVO == null) {
            throw new BaseException("用户信息不存在!");
        }
        //查询用户拓展信息
        UserCardVO userCardVO = userCardService.getByUserId(userId);
        if (userCardVO == null) {
            throw new BaseException("用户实名信息不存在!");
        }
        //拓展信息补充
        userVO.setRealName(userCardVO.getCardName());
        userVO.setBirthday(userCardVO.getBirthday());
        userVO.setSex(userCardVO.getSex());
        userVO.setIdCard(userCardVO.getCardId());
        userVO.setRegMobile(user.getShowPhoneNum());
        userVO.setMobile(user.getShowPhoneNum());
        userVO.setRegEmail(user.getEmail());
        userVO.setRegIp(user.getIp());
        userVO.setDeviceId(user.getMac());
        //注册游戏
        GameDTO gameDTO = gameService.getById(user.getGameId());
        if (gameDTO != null) {
            userVO.setRegGame(gameDTO.getName());
        }
        //渠道
        PromoChannel promoChannel = promoChannelService.getById(user.getAgentId());
        if (promoChannel != null) {
            userVO.setChannelName(promoChannel.getChannel());
            PromoAccount promoAccount = promoAccountService.getById(promoChannel.getAccountId());
            userVO.setRegFrom(promoAccount.getAccount());
        }
        //vip等级
        GameUserVO maxVipGameUser = gameUserService.getMaxVipGameUser(userId);
        if (maxVipGameUser != null) {
            userVO.setVipMax(maxVipGameUser.getRoleVipMax());
        }
        return userVO;
    }


    /**
     * 玩家相关条件查询用户信息
     *
     * @param userId      : 用户id
     * @param agentIdList : 渠道id列表
     * @param userName    : 用户名称
     * @param nickname    : 用户昵称
     * @param regIp       : 用户注册ip
     * @param adId        : 广告id
     * @return : 返回用户信息map
     */
    @Override
    public Map<Long, UserDTO> userCondition(Long userId, List<Long> agentIdList, String userName, String nickname,
                                            String regIp, String adId) {
        //用户信息条件处理
        if (userId == null && CollectionUtils.isEmpty(agentIdList) && Strings.isBlank(userName)
                && Strings.isBlank(nickname) && Strings.isBlank(regIp) && Strings.isBlank(adId)) {
            return null;
        }
        //查询
        List<User> userList = super.list(new LambdaQueryWrapper<User>()
                .eq(userId != null, User::getId, userId)
                .eq(Strings.isNotBlank(adId), User::getAdId, adId)
                .in(CollectionUtils.isNotEmpty(agentIdList), User::getAgentId, agentIdList)
                .eq(Strings.isNotBlank(regIp), User::getIp, regIp)
                .like(Strings.isNotBlank(userName), User::getUsername, userName)
                .like(Strings.isNotBlank(nickname), User::getNickname, nickname)
                .select(User::getId, User::getUsername, User::getNickname, User::getIp, User::getCreateTime, User::getAgentId, User::getAdId));
        if (CollectionUtils.isEmpty(userList)) {
            return Collections.emptyMap();
        }
        List<UserDTO> list = BeanUtils.copyList(userList, UserDTO.class);
        //构造返回
        return list.stream().collect(Collectors.toMap(UserDTO::getId, Function.identity()));
    }

    /**
     * 根据用户id查询信息
     *
     * @param userId : 返回用户信息
     * @return : 返回用户信息
     */
    @Override
    public UserDTO getById(Long userId) {
        return BeanUtils.copy(super.getById(userId), UserDTO.class);
    }

}
