package com.zanxiang.manage.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.zanxiang.manage.domain.dto.UserDTO;
import com.zanxiang.manage.domain.params.UserListParam;
import com.zanxiang.manage.domain.params.UserUpdateParam;
import com.zanxiang.manage.domain.vo.UserListVO;
import com.zanxiang.manage.domain.vo.UserVO;
import com.zanxiang.mybatis.entity.User;

import java.util.List;
import java.util.Map;

/**
 * @author : lingfeng
 * @time : 2022-06-08
 * @description : 用户信息
 */
public interface UserService extends IService<User> {

    /**
     * 玩家列表查询
     *
     * @param param : 查询参数
     * @return : 返回单页玩家数据
     */
    IPage<UserListVO> list(UserListParam param);

    /**
     * 用户信息更新
     *
     * @param param : 用户更新参数
     * @return : 返回更新结果
     */
    Boolean userUpdate(UserUpdateParam param);

    /**
     * 查询玩家详细信息
     *
     * @param userId : 用户id
     * @return : 返回用户信息
     */
    UserVO getUserInfo(Long userId);

    /**
     * 玩家相关条件查询用户信息
     *
     * @param userId      : 用户id
     * @param agentIdList : 渠道id列表
     * @param userName    : 用户名称
     * @param nickname    : 用户昵称
     * @param regIp       : 用户注册ip
     * @param adId        : 广告id
     * @return : 返回用户信息map
     */
    Map<Long, UserDTO> userCondition(Long userId, List<Long> agentIdList, String userName, String nickname, String regIp, String adId);

    /**
     * 根据用户id查询信息
     *
     * @param userId : 返回用户信息
     * @return : 返回用户信息
     */
    UserDTO getById(Long userId);
}
