package com.zanxiang.sdk.controller;

import com.zanxiang.common.domain.ResultVO;
import com.zanxiang.sdk.annotation.ValidLogin;
import com.zanxiang.sdk.domain.params.*;
import com.zanxiang.sdk.domain.vo.CustomerVO;
import com.zanxiang.sdk.domain.vo.UserLoginVO;
import com.zanxiang.sdk.domain.vo.UserVO;
import com.zanxiang.sdk.service.GameUserRoleService;
import com.zanxiang.sdk.service.UserCardService;
import com.zanxiang.sdk.service.UserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

/**
 * @author : lingfeng
 * @time : 2022-06-16
 * @description : 用户接口
 */
@Api(tags = "用户接口")
@RestController
@RequestMapping(value = "/api/user")
public class UserController {

    @Autowired
    private UserService userService;

    @Autowired
    private UserCardService userCardService;

    @Autowired
    private GameUserRoleService gameUserRoleService;

    @ApiOperation(value = "忘记密码找回")
    @PostMapping("/find/password")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = UserLoginVO.class)})
    public ResultVO<Boolean> findPassword(@Validated @RequestBody FindPasswordParam param) {
        return userService.findPassword(param);
    }

    @ApiOperation(value = "用户修改密码")
    @PostMapping("/update/password")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = UserLoginVO.class)})
    public ResultVO<Boolean> updatePassword(@Validated @RequestBody UpdatePasswordParam param, @ValidLogin UserData userData) {
        return userService.updatePassword(param, userData);
    }

    @ApiOperation(value = "用户绑定手机")
    @PostMapping("/bind/phone")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = UserLoginVO.class)})
    public ResultVO<Boolean> bindPhone(@Validated @RequestBody BindPhoneParam param, @ValidLogin UserData userData) {
        return userService.bindPhone(param, userData);
    }

    @ApiOperation(value = "用户实名认证")
    @PostMapping("/authentication")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = UserLoginVO.class)})
    public ResultVO<Boolean> userAuthentication(@Validated @RequestBody UserCardUpdateParam param, @ValidLogin UserData userData) {
        return userCardService.userAuthentication(param, userData);
    }

    @ApiOperation(value = "获取用户信息")
    @GetMapping("/get/detail")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = UserVO.class)})
    public ResultVO<UserVO> getUserDetail(@ValidLogin UserData userData) {
        return ResultVO.ok(userService.getUserDetail(userData));
    }

    @ApiOperation(value = "获取客服信息")
    @GetMapping("/get/customer")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = CustomerVO.class)})
    public ResultVO<CustomerVO> getUserCustomer(UserData userData) {
        return ResultVO.ok(userService.getUserCustomer(userData));
    }

    @ApiOperation(value = "上传角色信息")
    @PostMapping("/update/game/role")
    @ApiResponses(value = {@ApiResponse(code = 200, message = "成功", response = UserLoginVO.class)})
    public ResultVO updateUserGameRole(@Validated @RequestBody GameUserRoleUpdateParam param, @ValidLogin UserData userData) {
        return ResultVO.ok(gameUserRoleService.updateUserGameRole(param, userData));
    }

}
