package com.zanxiang.sdk.listener;

import com.zanxiang.common.enums.OrderStateEnum;
import com.zanxiang.common.utils.StringUtils;
import com.zanxiang.sdk.domain.dto.PlatformOrderDTO;
import com.zanxiang.sdk.service.PerformOrderService;
import com.zanxiang.sdk.service.PlatformOrderService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.event.EventListener;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Objects;

/**
 * 订单支付完成事件监听
 *
 * @author xufeng
 * @date 2022/6/27 10:22
 */
@Configuration
@Component
@Slf4j
public class OrderPaySuccessListener {

    @Resource
    private PlatformOrderService platformOrderService;

    @Resource
    private PerformOrderService performOrderService;

    @EventListener
    @Async
    public void onApplicationEvent(OrderPaySuccessEvent event) {
        if (StringUtils.isEmpty(event.getOrderId())) {
            log.error("缺少订单ID");
            return;
        }
        log.info("订单:{} 支付成功履约监听逻辑 ------start---------", event.getOrderId());
        PlatformOrderDTO orderInfo = platformOrderService.info(event.getOrderId());
        if (Objects.isNull(orderInfo)) {
            log.error("订单:{} 获取详情失败", event.getOrderId());
            return;
        }
        //--------非成功支付状态，仅操作宽表信息----------------
        if (!orderInfo.getStatus().equals(OrderStateEnum.SUCCESS.getCode())) {
            //生成宽表数据
            performOrderService.createOrderComplete(orderInfo);
            log.info("订单:{} 支付成功履约监听逻辑 ------end---------", event.getOrderId());
            return;
        }
        //--------成功支付操作----------------

        //检查是否为首充（账号首充）
        Boolean isFirstRecharge = performOrderService.checkIsFirstRecharge(orderInfo);
        //cp推送
        performOrderService.pushCp(orderInfo);
        //生成宽表数据
        performOrderService.createOrderComplete(orderInfo);
        //用户充值统计更新
        performOrderService.userRechargeTotal(orderInfo);
        //商户号额度统计更新
        performOrderService.payMerchantTotal(orderInfo);
        

        log.info("订单:{} 支付成功履约监听逻辑 ------end---------", event.getOrderId());
    }
}
