package com.zanxiang.sdk.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.enums.ExpireTimeEnum;
import com.zanxiang.mybatis.entity.GameExt;
import com.zanxiang.mybatis.mapper.GameExtMapper;
import com.zanxiang.sdk.constant.RedisKeyConstant;
import com.zanxiang.sdk.service.GameExtService;
import com.zanxiang.sdk.util.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 * @author : lingfeng
 * @time : 2022-09-23
 * @description : 游戏密钥
 */
@Slf4j
@Service
public class GameExtServiceImpl extends ServiceImpl<GameExtMapper, GameExt> implements GameExtService {

    @Autowired
    private RedisUtil<GameExt> redisUtil;

    /**
     * 根据游戏id查询密钥
     *
     * @param gameId : 游戏id
     * @return : 返回游戏密钥
     */
    @Override
    public GameExt getByGameId(Long gameId) {
        //查询缓存
        GameExt gameKey = redisUtil.getCache(this.getGameKey(String.valueOf(gameId)));
        if (gameKey == null) {
            gameKey = this.getOne(new LambdaQueryWrapper<GameExt>().eq(GameExt::getGameId, gameId));
        }
        if (gameKey != null) {
            redisUtil.setCache(this.getGameKey(String.valueOf(gameId)), gameKey, ExpireTimeEnum.ONE_DAY.getTime());
        }
        return gameKey;
    }

    /**
     * 根据游戏appId查询密钥
     *
     * @param appId : 游戏appId
     * @return : 返回游戏密钥
     */
    @Override
    public GameExt getByGameAppId(String appId) {
        //查询缓存
        GameExt gameKey = redisUtil.getCache(this.getGameKey(appId));
        if (gameKey == null) {
            gameKey = this.getOne(new LambdaQueryWrapper<GameExt>().eq(GameExt::getAppId, appId));
        }
        if (gameKey != null) {
            redisUtil.setCache(this.getGameKey(appId), gameKey, ExpireTimeEnum.ONE_DAY.getTime());
        }
        return gameKey;
    }

    /**
     * 拼接游戏缓存key
     *
     * @param key : 游戏标识
     * @return : 返回游戏key的缓存key
     */
    private String getGameKey(String key) {
        return RedisKeyConstant.GAME_SECRET_KEY + "_" + key;
    }
}
