package com.zanxiang.sdk.service.Impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zanxiang.common.enums.OrderStateEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.utils.bean.BeanUtils;
import com.zanxiang.mybatis.entity.Order;
import com.zanxiang.mybatis.mapper.OrderMapper;
import com.zanxiang.sdk.domain.bo.PlatformOrderBO;
import com.zanxiang.sdk.domain.dto.PlatformOrderDTO;
import com.zanxiang.sdk.listener.OrderPaySuccessEvent;
import com.zanxiang.sdk.service.PlatformOrderService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;

/**
 * 平台订单逻辑开发
 *
 * @author xufeng
 * @date 2022-06-06 17:06
 */
@Slf4j
@Service
@Transactional(rollbackFor = {Exception.class, RuntimeException.class})
public class PlatformOrderServiceImpl extends ServiceImpl<OrderMapper, Order> implements PlatformOrderService {

    @Autowired
    private ApplicationContext applicationContext;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String create(PlatformOrderBO platformOrderBO) {
        try {
            log.info("生成订单请求参数 platformOrderBO:{}", platformOrderBO);
            Order data = BeanUtils.copy(platformOrderBO, Order.class);
            if (save(data)) {
                OrderPaySuccessEvent orderPaySuccessEvent = new OrderPaySuccessEvent(this, data.getId());
                applicationContext.publishEvent(orderPaySuccessEvent);
                return data.getId();
            }
        } catch (Exception e) {
            log.error("生成订单失败 platformOrderBO:{}, error:{}", platformOrderBO, e);
        }
        return null;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean pay(PlatformOrderBO platformOrderBO) {
        log.info("订单支付请求参数 platformOrderBO:{}", platformOrderBO);
        try {
            Order order = getOne(new LambdaQueryWrapper<Order>().eq(Order::getId, platformOrderBO.getId()));
            //已支付情况，直接返回成功
            if (order.getStatus().equals(OrderStateEnum.SUCCESS.getCode())) {
                return true;
            }
            if (!order.getStatus().equals(OrderStateEnum.NO_PAY.getCode())) {
                throw new BaseException("订单状态非待支付");
            }
            platformOrderBO.setPayTime(LocalDateTime.now());
            log.info("订单支付提交数据 data:{}", platformOrderBO);
            boolean result = updateById(BeanUtils.copy(platformOrderBO, Order.class));
            if (result) {
                OrderPaySuccessEvent orderPaySuccessEvent = new OrderPaySuccessEvent(this, order.getId());
                applicationContext.publishEvent(orderPaySuccessEvent);
                return true;
            }
            return false;
        } catch (Exception e) {
            log.error("订单支付失败 platformOrderBO:{}, error:{}", platformOrderBO, e);
            return false;
        }
    }

    @Override
    public PlatformOrderDTO info(String id) {
        Order order = getOne(new LambdaQueryWrapper<Order>().eq(Order::getId, id));
        return BeanUtils.copy(order, PlatformOrderDTO.class);
    }
}
