package com.zanxiang.sdk.service;

import com.zanxiang.common.domain.ResultVO;
import com.zanxiang.sdk.domain.params.LoginMobileParam;
import com.zanxiang.sdk.domain.params.LoginPasswordParam;
import com.zanxiang.sdk.domain.params.QqLoginCallbackParam;
import com.zanxiang.sdk.domain.params.UserData;
import com.zanxiang.sdk.domain.vo.UserLoginVO;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * @author : lingfeng
 * @time : 2022-06-22
 * @description :
 */
public interface RegisterLoginService {

    /**
     * 用户名注册或者登录
     *
     * @param param    : 注册登录参数
     * @param userData : 用户信息
     * @return : 返回用户登录信息
     */
    ResultVO<UserLoginVO> loginPassword(LoginPasswordParam param, UserData userData);

    /**
     * 手机号登录
     *
     * @param param    : 手机号登录参数
     * @param userData : 用户信息
     * @return : 返回登录信息
     */
    ResultVO<UserLoginVO> loginMobile(LoginMobileParam param, UserData userData);

    /**
     * 登录退出
     *
     * @param userData : 返回用户信息
     * @return : 返回结果
     */
    Boolean loginOut(UserData userData);

    /**
     * 登录注销
     *
     * @param userData : 返回用户信息
     * @return : 返回结果
     */
    Boolean writeOff(UserData userData);

    /**
     * QQ授权登录
     *
     * @param response : 返回体
     */
    void qqLoginAuth(HttpServletResponse response);

    /**
     * QQ授权注册登录回调
     *
     * @param qqLoginCallbackParam : 回调参数
     * @param request              : request
     * @param userData             : userData
     * @return : 返回登录token
     */
    ResultVO<UserLoginVO> qqLoginCallback(QqLoginCallbackParam qqLoginCallbackParam, HttpServletRequest request, UserData userData);

    /**
     * 微信授权登录
     *
     * @param response : 返回体
     */
    void vxLoginAuth(HttpServletResponse response);

    /**
     * 微信授权注册登录回调
     *
     * @param qqLoginCallbackParam : 回调参数
     * @param request              : request
     * @param userData             : userData
     * @return : 返回登录token
     */
    ResultVO<UserLoginVO> vxLoginCallback(QqLoginCallbackParam qqLoginCallbackParam, HttpServletRequest request, UserData userData);
}
