package com.zanxiang.sdk.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.zanxiang.common.domain.ResultVO;
import com.zanxiang.mybatis.entity.User;
import com.zanxiang.sdk.domain.dto.UserDTO;
import com.zanxiang.sdk.domain.params.BindPhoneParam;
import com.zanxiang.sdk.domain.params.FindPasswordParam;
import com.zanxiang.sdk.domain.params.UpdatePasswordParam;
import com.zanxiang.sdk.domain.params.UserData;
import com.zanxiang.sdk.domain.vo.CustomerVO;
import com.zanxiang.sdk.domain.vo.UserVO;

/**
 * @author : lingfeng
 * @time : 2022-06-08
 * @description : 用户信息
 */
public interface UserService extends IService<User> {

    /**
     * 忘记密码找回
     *
     * @param param : 修改密码的参数
     * @return : 返回修改结果
     */
    ResultVO<Boolean> findPassword(FindPasswordParam param);

    /**
     * 用户重置密码
     *
     * @param updatePasswordParam : 修改密码的参数
     * @param userData            : userData
     * @return : 返回修改结果
     */
    ResultVO<Boolean> updatePassword(UpdatePasswordParam updatePasswordParam, UserData userData);

    /**
     * 用户绑定手机
     *
     * @param param    : 绑定手机参数
     * @param userData : userData
     * @return : 返回修改结果
     */
    ResultVO<Boolean> bindPhone(BindPhoneParam param, UserData userData);

    /**
     * 获取用户信息
     *
     * @param userData : 用户信息
     * @return : 返回用户详情信息
     */
    UserVO getUserDetail(UserData userData);

    /**
     * 用户获取游戏客服信息
     *
     * @param userData : userData
     * @return : 返回用户信息
     */
    CustomerVO getUserCustomer(UserData userData);

    /**
     * 根据手机号获取用户信息
     *
     * @param mobile : 用户手机号
     * @return : 返回用户信息
     */
    UserDTO getUserInfoByMobile(String mobile);

}
