package com.zanxiang.sdk.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.zanxiang.common.domain.ResultVO;
import com.zanxiang.mybatis.entity.UserToken;
import com.zanxiang.sdk.domain.dto.UserTokenDTO;
import com.zanxiang.sdk.domain.params.UserData;

/**
 * @author : lingfeng
 * @time : 2022-06-08
 * @description : 用户token管理
 */
public interface UserTokenService extends IService<UserToken> {

    /**
     * 登录凭证验证
     *
     * @param appId  : 游戏id
     * @param userId : 用户id
     * @param token  : 用户token
     * @param sign   : 签名
     * @return : 返回验证结果
     */
    ResultVO<Long> userTokenCheck(String appId, Long userId, String token, String sign);

    /**
     * 登录凭证验证
     *
     * @param userData : 用户信息
     * @return : 返回凭证是否过期
     */
    Boolean userTokenExpireTimeCheck(UserData userData);

    /**
     * 通过token设备获取token详情
     *
     * @param token      : 用户token
     * @param deviceType : 设备类型
     * @return : 返回用户token详情
     */
    UserTokenDTO getTokenInfoByTokenDevice(String token, Integer deviceType);

    /**
     * 获取用户token
     *
     * @param userId     : 用户id
     * @param deviceType : 设备类型
     * @return : 返回token
     */
    String getUserToken(Long userId, Integer deviceType);

    /**
     * 删除用户token
     *
     * @param userId     : 用户id
     * @param deviceType : 设备类型
     */
    void deleteUserToken(Long userId, Integer deviceType);
}
